from datetime import datetime
from enum import Enum

from checkout_sdk.common.common import Phone, Address
from checkout_sdk.common.enums import Currency, ChallengeIndicator, CardholderAccountAgeIndicatorType, \
    AccountChangeIndicatorType, AccountPasswordChangeIndicatorType, AccountTypeCardProductType


class ChannelType(str, Enum):
    APP = 'app'
    BROWSER = 'browser'
    MERCHANT_INITIATED = 'merchant_initiated'


class SdkInterfaceType(str, Enum):
    BOTH = 'both'
    HTML = 'html'
    NATIVE = 'native'


class ThreeDsMethodCompletion(str, Enum):
    Y = 'y'
    N = 'n'
    U = 'u'


class CompletionInfoType(str, Enum):
    HOSTED = 'hosted'
    NON_HOSTED = 'non_hosted'


class SessionSourceType(str, Enum):
    CARD = 'card'
    ID = 'id'
    TOKEN = 'token'
    NETWORK_TOKEN = 'network_token'


class AuthenticationType(str, Enum):
    ADD_CARD = 'add_card'
    INSTALLMENT = 'installment'
    MAINTAIN_CARD = 'maintain_card'
    RECURRING = 'recurring'
    REGULAR = 'regular'


class Category(str, Enum):
    PAYMENT = 'payment'
    NON_PAYMENT = 'nonPayment'


class TransactionType(str, Enum):
    ACCOUNT_FUNDING = 'account_funding'
    CHECK_ACCEPTANCE = 'check_acceptance'
    GOODS_SERVICE = 'goods_service'
    PREPAID_ACTIVATION_AND_LOAD = 'prepaid_activation_and_load'
    QUASHI_CARD_TRANSACTION = 'quashi_card_transaction'


class UIElements(str, Enum):
    TEXT = 'text'
    SINGLE_SELECT = 'single_select'
    MULTI_SELECT = 'multi_select'
    OOB = 'oob'
    HTML_OTHER = 'html_other'


class SessionScheme(str, Enum):
    VISA = 'visa'
    MASTERCARD = 'mastercard'
    JCB = 'jcb'
    AMEX = 'amex'
    DINERS = 'diners'
    CARTES_BANCAIRES = 'cartes_bancaires'


class AuthenticationMethod(str, Enum):
    FEDERATED_ID = 'federated_id'
    FIDO = 'fido'
    ISSUER_CREDENTIALS = 'issuer_credentials'
    NO_AUTHENTICATION = 'no_authentication'
    OWN_CREDENTIALS = 'own_credentials'
    THIRD_PARTY_AUTHENTICATION = 'third_party_authentication'


class DeliveryTimeframe(str, Enum):
    ELECTRONIC_DELIVERY = 'electronic_delivery'
    SAME_DAY = 'same_day'
    OVERNIGHT = 'overnight'
    TWO_DAY_OR_MORE = 'two_day_or_more'


class ShippingIndicator(str, Enum):
    VISA = 'visa'


class SdkEphemeralPublicKey:
    kty: str
    crv: str
    x: str
    y: str


class SessionAddress(Address):
    address_line3: str


class SessionMarketplaceData:
    sub_entity_id: str


class SessionsBillingDescriptor:
    name: str


# Channel
class ChannelData:
    channel: ChannelType

    def __init__(self, channel_p: ChannelType):
        self.channel = channel_p


class AppSession(ChannelData):
    sdk_app_id: str
    sdk_max_timeout: int
    sdk_ephem_pub_key: SdkEphemeralPublicKey
    sdk_reference_number: str
    sdk_encrypted_data: str
    sdk_transaction_id: str
    sdk_interface_type: SdkInterfaceType
    sdk_ui_elements: list  # UIElements

    def __init__(self):
        super().__init__(ChannelType.APP)


class BrowserSession(ChannelData):
    three_ds_method_completion: ThreeDsMethodCompletion = ThreeDsMethodCompletion.U
    accept_header: str
    java_enabled: bool
    javascript_enabled: bool
    language: str
    color_depth: str
    screen_height: str
    screen_width: str
    timezone: str
    user_agent: str
    ip_address: str

    def __init__(self):
        super().__init__(ChannelType.BROWSER)


class RequestType(str, Enum):
    ACCOUNT_VERIFICATION = "account_verification"
    ADD_CARD = "add_card"
    INSTALLMENT_TRANSACTION = "installment_transaction"
    MAIL_ORDER = "mail_order"
    MAINTAIN_CARD_INFORMATION = "maintain_card_information"
    OTHER_PAYMENT = "other_payment"
    RECURRING_TRANSACTION = "recurring_transaction"
    SPLIT_OR_DELAYED_SHIPMENT = "split_or_delayed_shipment"
    TELEPHONE_ORDER = "telephone_order"
    TOP_UP = "top_up"
    WHITELIST_STATUS_CHECK = "whitelist_status_check"


class MerchantInitiatedSession(ChannelData):
    request_type: RequestType

    def __init__(self):
        super().__init__(ChannelType.MERCHANT_INITIATED)


# Completion
class Completion:
    type: CompletionInfoType

    def __init__(self, type_p: CompletionInfoType):
        self.type = type_p


class HostedCompletionInfo(Completion):
    callback_url: str
    success_url: str
    failure_url: bool

    def __init__(self):
        super().__init__(CompletionInfoType.HOSTED)


class NonHostedCompletionInfo(Completion):
    callback_url: str
    challenge_notification_url: str

    def __init__(self):
        super().__init__(CompletionInfoType.NON_HOSTED)


# Source
class SessionSource:
    type: SessionSourceType
    scheme: SessionScheme
    billing_address: SessionAddress
    home_phone: Phone
    mobile_phone: Phone
    work_phone: Phone
    email: str

    def __init__(self, type_p: SessionSourceType):
        self.type = type_p


class SessionCardSource(SessionSource):
    number: str
    expiry_month: int
    expiry_year: int
    name: str
    stored: bool = False
    store_for_future_use: bool

    def __init__(self):
        super().__init__(SessionSourceType.CARD)


class SessionIdSource(SessionSource):
    id: str

    def __init__(self):
        super().__init__(SessionSourceType.ID)


class SessionTokenSource(SessionSource):
    token: str
    store_for_future_use: bool

    def __init__(self):
        super().__init__(SessionSourceType.TOKEN)


class NetworkTokenSource(SessionSource):
    token: str
    expiry_month: int
    expiry_year: int
    name: str
    stored: bool

    def __init__(self):
        super().__init__(SessionSourceType.NETWORK_TOKEN)


class ThreeDsReqAuthMethod(str, Enum):
    NoThreedsRequestorAuthenticationOccurred = "no_threeds_requestor_authentication_occurred"
    Three3dsRequestorOwnCredentials = "three3ds_requestor_own_credentials"
    FederatedId = "federated_id"
    IssuerCredentials = "issuer_credentials"
    ThirdPartyAuthentication = "third_party_authentication"
    FidoAuthenticator = "fido_authenticator"
    FidoAuthenticatorFidoAssuranceDataSigned = "fido_authenticator_fido_assurance_data_signed"
    SrcAssuranceData = "src_assurance_data"


class ThreeDsRequestorAuthenticationInfo:
    three_ds_req_auth_method: ThreeDsReqAuthMethod
    three_ds_req_auth_timestamp: datetime
    three_ds_req_auth_data: str


class CardholderAccountInfo:
    purchase_count: int
    account_age: str
    add_card_attempts: int
    shipping_address_age: str
    account_name_matches_shipping_name: bool
    suspicious_account_activity: bool
    transactions_today: int
    # @deprecated This property will be removed in the future, and should not be used.
    authentication_method: AuthenticationMethod
    cardholder_account_age_indicator: CardholderAccountAgeIndicatorType
    account_change: datetime
    account_change_indicator: AccountChangeIndicatorType
    account_date: datetime
    account_password_change: str
    account_password_change_indicator: AccountPasswordChangeIndicatorType
    transactions_per_year: int
    payment_account_age: datetime
    shipping_address_usage: datetime
    account_type: AccountTypeCardProductType
    account_id: str
    three_ds_requestor_authentication_info: ThreeDsRequestorAuthenticationInfo


class ReorderItemsIndicatorType(str, Enum):
    FIRST_TIME_ORDERED = "first_time_ordered"
    REORDERED = "reordered"


class PreOrderPurchaseIndicatorType(str, Enum):
    FUTURE_AVAILABILITY = "future_availability"
    MERCHANDISE_AVAILABLE = "merchandise_available"


class MerchantRiskInfo:
    delivery_email: str
    delivery_timeframe: DeliveryTimeframe
    is_preorder: bool
    is_reorder: bool
    shipping_indicator: ShippingIndicator
    reorder_items_indicator: ReorderItemsIndicatorType
    pre_order_purchase_indicator: PreOrderPurchaseIndicatorType
    pre_order_date: datetime
    gift_card_amount: str
    gift_card_currency: str
    gift_card_count: str


class Recurring:
    days_between_payments: int = 1
    expiry: str = '99991231'


class Installment:
    number_of_payments: int
    days_between_payments: int = 1
    expiry: str = '99991231'


class OptimizedProperties:
    field: str
    original_value: str
    optimized_value: str


class Optimization:
    optimized: bool
    framework: str
    optimized_properties: list  # OptimizedProperties


class InitialTransaction:
    acs_transaction_id: str
    authentication_method: str
    authentication_timestamp: str
    authentication_data: str
    initial_session_id: str


class SessionRequest:
    source: SessionSource = SessionCardSource()
    amount: int
    currency: Currency
    processing_channel_id: str
    marketplace: SessionMarketplaceData
    authentication_type: AuthenticationType = AuthenticationType.REGULAR
    authentication_category: Category = Category.PAYMENT
    account_info: CardholderAccountInfo
    challenge_indicator: ChallengeIndicator = ChallengeIndicator.NO_PREFERENCE
    billing_descriptor: SessionsBillingDescriptor
    reference: str
    merchant_risk_info: MerchantRiskInfo
    prior_transaction_reference: str
    transaction_type: TransactionType = TransactionType.GOODS_SERVICE
    shipping_address: SessionAddress
    shipping_address_matches_billing: bool
    completion: Completion
    channel_data: ChannelData
    recurring: Recurring
    installment: Installment
    optimization: Optimization
    initial_transaction: InitialTransaction


class ThreeDsMethodCompletionRequest:
    three_ds_method_completion: ThreeDsMethodCompletion



from checkout_sdk.common.common import Address, CustomerRequest, Phone
from checkout_sdk.common.enums import InstrumentType


class InstrumentAccountHolder:
    billing_address: Address
    phone: Phone


class InstrumentCustomerRequest(CustomerRequest):
    default: bool
    phone: Phone


class CreateInstrumentRequest:
    type: InstrumentType
    token: str
    account_holder: InstrumentAccountHolder
    customer: InstrumentCustomerRequest

    def __init__(self):
        self.type = InstrumentType.TOKEN


class UpdateInstrumentCustomer:
    id: str
    default: bool


class UpdateInstrumentRequest:
    expiry_month: int
    expiry_year: int
    name: str
    account_holder: InstrumentAccountHolder
    customer: UpdateInstrumentCustomer



from __future__ import absolute_import

from enum import Enum


class AuthorizationType(Enum):
    PUBLIC_KEY = 'PUBLIC_KEY'
    SECRET_KEY = 'SECRET_KEY'
    PUBLIC_KEY_OR_OAUTH = 'PUBLIC_KEY_OR_OAUTH'
    SECRET_KEY_OR_OAUTH = 'SECRET_KEY_OR_OAUTH'
    OAUTH = 'OAUTH'
    CUSTOM = 'CUSTOM'



from __future__ import absolute_import

from abc import ABCMeta

from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.checkout_sdk_builder import CheckoutSdkBuilder
from checkout_sdk.checkout_api import CheckoutApi
from checkout_sdk.oauth_credentials import OAuthSdkCredentials


class OAuthSdk(CheckoutSdkBuilder, metaclass=ABCMeta):
    _client_id: str = ''
    _client_secret: str = ''
    _authorization_uri: str = ''
    _scopes: list = []

    def __init__(self):
        super().__init__()

    def client_credentials(self, client_id: str, client_secret: str):
        self._client_id = client_id
        self._client_secret = client_secret
        return self

    def authorization_uri(self, authorization_uri: str):
        self._authorization_uri = authorization_uri
        return self

    def scopes(self, scopes: list):
        self._scopes = scopes
        return self

    def build(self):
        if self._environment_subdomain is not None:
            configuration = CheckoutConfiguration(
                credentials=OAuthSdkCredentials.init(http_client=self._http_client,
                                                     environment=self._environment,
                                                     client_id=self._client_id,
                                                     client_secret=self._client_secret,
                                                     scopes=self._scopes,
                                                     authorization_uri=self._authorization_uri),
                environment=self._environment,
                http_client=self._http_client,
                environment_subdomain=self._environment_subdomain)
        else:
            configuration = CheckoutConfiguration(
                credentials=OAuthSdkCredentials.init(http_client=self._http_client,
                                                     environment=self._environment,
                                                     client_id=self._client_id,
                                                     client_secret=self._client_secret,
                                                     scopes=self._scopes,
                                                     authorization_uri=self._authorization_uri),
                environment=self._environment,
                http_client=self._http_client)
        return CheckoutApi(configuration)



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.apm.klarna import CreditSessionRequest, OrderCaptureRequest
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.payments.payments import VoidRequest


class KlarnaClient(Client):
    __CREDIT_SESSIONS_PATH = 'credit-sessions'
    __ORDERS_PATH = 'orders'
    __CAPTURES_PATH = 'captures'
    __VOIDS_PATH = 'voids'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def create_credit_session(self, create_session_request: CreditSessionRequest):
        return self._api_client.post(self.build_path(self.__get_base_url(), self.__CREDIT_SESSIONS_PATH),
                                     self._sdk_authorization(), create_session_request)

    def get_credit_session(self, session_id: str):
        return self._api_client.get(self.build_path(self.__get_base_url(), self.__CREDIT_SESSIONS_PATH, session_id),
                                    self._sdk_authorization())

    def capture_payment(self, payment_id: str, order_capture_request: OrderCaptureRequest):
        return self._api_client.post(
            self.build_path(self.__get_base_url(), self.__ORDERS_PATH, payment_id, self.__CAPTURES_PATH),
            self._sdk_authorization(), order_capture_request)

    def void_payment(self, payment_id: str, void_request: VoidRequest):
        return self._api_client.post(
            self.build_path(self.__get_base_url(), self.__ORDERS_PATH, payment_id, self.__VOIDS_PATH),
            self._sdk_authorization(), void_request)

    def __get_base_url(self):
        return 'klarna-external' if self.is_sandbox() else 'klarna'



from __future__ import absolute_import

from typing import Optional

from checkout_sdk.default_http_client import DefaultHttpClientBuilder
from checkout_sdk.environment import Environment
from checkout_sdk.environment_subdomain import EnvironmentSubdomain
from checkout_sdk.http_client_interface import HttpClientBuilderInterface


class CheckoutSdkBuilder:

    def __init__(self):
        self._environment = Environment.sandbox()
        self._environment_subdomain = None
        self._http_client = DefaultHttpClientBuilder().get_client()

    def environment(self, environment: Environment):
        self._environment = environment
        return self

    def environment_subdomain(self, subdomain: Optional[str]):
        if subdomain:
            self._environment_subdomain = EnvironmentSubdomain(self._environment, subdomain)
        else:
            self._environment_subdomain = None
        return self

    def http_client_builder(self, http_client_builder: HttpClientBuilderInterface):
        self._http_client = http_client_builder.get_client()
        return self

    def build(self):
        raise NotImplementedError()



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.sources.sources import SepaSourceRequest


class SourcesClient(Client):

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def create_sepa_source(self, sepa_source_request: SepaSourceRequest):
        return self._api_client.post('sources', self._sdk_authorization(), sepa_source_request)



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.webhooks.webhooks import WebhookRequest


class WebhooksClient(Client):
    __WEBHOOKS_PATH = 'webhooks'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def retrieve_webhooks(self):
        return self._api_client.get(self.__WEBHOOKS_PATH, self._sdk_authorization())

    def register_webhook(self, webhook_request: WebhookRequest):
        return self._api_client.post(self.__WEBHOOKS_PATH, self._sdk_authorization(), webhook_request)

    def retrieve_webhook(self, webhook_id: str):
        return self._api_client.get(self.build_path(self.__WEBHOOKS_PATH, webhook_id), self._sdk_authorization())

    def update_webhook(self, webhook_id: str, webhook_request: WebhookRequest):
        return self._api_client.put(self.build_path(self.__WEBHOOKS_PATH, webhook_id),
                                    self._sdk_authorization(), webhook_request)

    def patch_webhook(self, webhook_id: str, webhook_request: WebhookRequest):
        return self._api_client.patch(self.build_path(self.__WEBHOOKS_PATH, webhook_id),
                                      self._sdk_authorization(), webhook_request)

    def remove_webhook(self, webhook_id: str):
        return self._api_client.delete(self.build_path(self.__WEBHOOKS_PATH, webhook_id), self._sdk_authorization())



from __future__ import absolute_import

from datetime import datetime

from checkout_sdk.common.common import Address, AccountHolder
from checkout_sdk.common.enums import PaymentSourceType, Country, Currency, AccountType
from checkout_sdk.payments.payments import PaymentRequestSource, BillingPlan, PaymentMethodDetails
from checkout_sdk.tokens.tokens import ApplePayTokenData


class RequestIdealSource(PaymentRequestSource):
    description: str
    language: str

    def __init__(self):
        super().__init__(PaymentSourceType.IDEAL)


class RequestSofortSource(PaymentRequestSource):
    countryCode: Country
    languageCode: str

    def __init__(self):
        super().__init__(PaymentSourceType.SOFORT)


class RequestTamaraSource(PaymentRequestSource):
    billing_address: Address

    def __init__(self):
        super().__init__(PaymentSourceType.TAMARA)


class RequestPayPalSource(PaymentRequestSource):
    plan: BillingPlan

    def __init__(self):
        super().__init__(PaymentSourceType.PAYPAL)


class PaymentRequestWeChatPaySource(PaymentRequestSource):
    billing_address: Address

    def __init__(self):
        super().__init__(PaymentSourceType.WECHATPAY)


class RequestAlipayPlusSource(PaymentRequestSource):

    def __init__(self, source_type: PaymentSourceType):
        super().__init__(source_type)

    @staticmethod
    def request_alipay_plus_source():
        return RequestAlipayPlusSource(PaymentSourceType.ALIPAY_PLUS)

    @staticmethod
    def request_alipay_plus_cn_source():
        return RequestAlipayPlusSource(PaymentSourceType.ALIPAY_CN)

    @staticmethod
    def request_alipay_plus_hk_source():
        return RequestAlipayPlusSource(PaymentSourceType.ALIPAY_HK)

    @staticmethod
    def request_alipay_plus_gcash_source():
        return RequestAlipayPlusSource(PaymentSourceType.GCASH)

    @staticmethod
    def request_alipay_plus_dana_source():
        return RequestAlipayPlusSource(PaymentSourceType.DANA)

    @staticmethod
    def request_alipay_plus_kakao_pay_source():
        return RequestAlipayPlusSource(PaymentSourceType.KAKAOPAY)

    @staticmethod
    def request_alipay_plus_true_money_source():
        return RequestAlipayPlusSource(PaymentSourceType.TRUEMONEY)

    @staticmethod
    def request_alipay_plus_tng_source():
        return RequestAlipayPlusSource(PaymentSourceType.TNG)


class RequestAfterPaySource(PaymentRequestSource):
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.AFTERPAY)


class RequestBenefitSource(PaymentRequestSource):

    def __init__(self):
        super().__init__(PaymentSourceType.BENEFIT)


class RequestEpsSource(PaymentRequestSource):
    purpose: str
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.EPS)


class RequestIllicadoSource(PaymentRequestSource):
    billing_address: Address

    def __init__(self):
        super().__init__(PaymentSourceType.ILLICADO)


class RequestGiropaySource(PaymentRequestSource):
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.GIROPAY)


class RequestMbwaySource(PaymentRequestSource):

    def __init__(self):
        super().__init__(PaymentSourceType.MBWAY)


class RequestQPaySource(PaymentRequestSource):
    quantity: int
    description: str
    language: str
    national_id: str

    def __init__(self):
        super().__init__(PaymentSourceType.QPAY)


class RequestBancontactSource(PaymentRequestSource):
    payment_country: Country
    account_holder_name: str
    billing_descriptor: str
    language: str

    def __init__(self):
        super().__init__(PaymentSourceType.BANCONTACT)


class RequestKnetSource(PaymentRequestSource):
    language: str
    user_defined_field1: str
    user_defined_field2: str
    user_defined_field3: str
    user_defined_field4: str
    user_defined_field5: str
    card_token: str
    ptlf: str
    token_type: str
    token_data: ApplePayTokenData
    payment_method_details: PaymentMethodDetails

    def __init__(self):
        super().__init__(PaymentSourceType.KNET)


class RequestMultiBancoSource(PaymentRequestSource):
    payment_country: Country
    account_holder_name: str
    billing_descriptor: str

    def __init__(self):
        super().__init__(PaymentSourceType.MULTIBANCO)


class RequestP24Source(PaymentRequestSource):
    payment_country: Country
    account_holder_name: str
    account_holder_email: str
    billing_descriptor: str

    def __init__(self):
        super().__init__(PaymentSourceType.P24)


class RequestPostFinanceSource(PaymentRequestSource):
    payment_country: Country
    account_holder_name: str
    billing_descriptor: str

    def __init__(self):
        super().__init__(PaymentSourceType.POSTFINANCE)


class RequestStcPaySource(PaymentRequestSource):
    def __init__(self):
        super().__init__(PaymentSourceType.STCPAY)


class RequestAlmaSource(PaymentRequestSource):
    billing_address: Address

    def __init__(self):
        super().__init__(PaymentSourceType.ALMA)


class RequestKlarnaSource(PaymentRequestSource):
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.KLARNA)


class RequestFawrySource(PaymentRequestSource):
    description: str
    customer_profile_id: str
    customer_mobile: str
    customer_email: str
    expires_on: datetime
    products: list  # FawryProduct

    def __init__(self):
        super().__init__(PaymentSourceType.FAWRY)


class RequestCvConnectSource(PaymentRequestSource):
    billing_address: Address

    def __init__(self):
        super().__init__(PaymentSourceType.CVCONNECT)


class RequestTrustlySource(PaymentRequestSource):
    billing_address: Address

    def __init__(self):
        super().__init__(PaymentSourceType.TRUSTLY)


class RequestSepaSource(PaymentRequestSource):
    country: Country
    account_number: str
    bank_code: str
    currency: Currency
    mandate_id: str
    date_of_signature: str
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.SEPA)


class RequestAchSource(PaymentRequestSource):
    account_type: AccountType
    country: Country
    account_number: str
    bank_code: str
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.ACH)


class RequestBizumSource(PaymentRequestSource):
    mobile_number: str

    def __init__(self):
        super().__init__(PaymentSourceType.BIZUM)


class RequestOctopusSource(PaymentRequestSource):
    def __init__(self):
        super().__init__(PaymentSourceType.OCTOPUS)


class RequestPlaidSource(PaymentRequestSource):
    token: str
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.PLAID)


class RequestSequraSource(PaymentRequestSource):
    billing_address: Address

    def __init__(self):
        super().__init__(PaymentSourceType.SEQURA)



from __future__ import absolute_import

from enum import Enum


class OAuthScopes(str, Enum):
    VAULT = 'vault'
    VAULT_INSTRUMENTS = 'vault:instruments'
    VAULT_TOKENIZATION = 'vault:tokenization'
    VAULT_CARD_METADATA = 'vault:card-metadata'
    GATEWAY = 'gateway'
    GATEWAY_PAYMENT = 'gateway:payment'
    GATEWAY_PAYMENT_DETAILS = 'gateway:payment-details'
    GATEWAY_PAYMENT_AUTHORIZATION = 'gateway:payment-authorizations'
    GATEWAY_PAYMENT_VOIDS = 'gateway:payment-voids'
    GATEWAY_PAYMENT_CAPTURES = 'gateway:payment-captures'
    GATEWAY_PAYMENT_REFUNDS = 'gateway:payment-refunds'
    FX = 'fx'
    PAYOUTS_BANK_DETAILS = 'payouts:bank-details'
    SESSIONS_APP = 'sessions:app'
    SESSIONS_BROWSER = 'sessions:browser'
    DISPUTES = 'disputes'
    DISPUTES_VIEW = 'disputes:view'
    DISPUTES_PROVIDE_EVIDENCE = 'disputes:provide-evidence'
    DISPUTES_ACCEPT = 'disputes:accept'
    MARKETPLACE = 'marketplace'
    ACCOUNTS = 'accounts'
    FLOW = 'flow'
    FLOW_WORKFLOWS = 'flow:workflows'
    FLOW_EVENTS = 'flow:events'
    FILES = 'files'
    FILES_RETRIEVE = 'files:retrieve'
    FILES_UPLOAD = 'files:upload'
    FILES_DOWNLOAD = 'files:download'
    TRANSFERS = 'transfers'
    TRANSFERS_CREATE = 'transfers:create'
    TRANSFERS_VIEW = 'transfers:view'
    BALANCES = 'balances'
    BALANCES_VIEW = 'balances:view'
    REPORTS = 'reports'
    REPORTS_VIEW = 'reports:view'
    FINANCIAL_ACTIONS = 'financial-actions'
    FINANCIAL_ACTIONS_VIEW = 'financial-actions:view'

    MIDDLEWARE = 'middleware'
    MIDDLEWARE_MERCHANTS_SECRET = 'middleware:merchants-secret'
    MIDDLEWARE_MERCHANTS_PUBLIC = 'middleware:merchants-public'
    ISSUING_CLIENT = 'issuing:client'
    ISSUING_CARD_MGMT = 'issuing:card-mgmt'
    ISSUING_CONTROLS_READ = 'issuing:controls-read'
    ISSUING_CONTROLS_WRITE = 'issuing:controls-write'

    PAYMENT_CONTEXTS = 'Payment Contexts'

    FORWARD = 'forward'



from __future__ import absolute_import


class Environment:

    def __init__(self, base_uri,
                 authorization_uri,
                 files_uri,
                 transfers_uri,
                 balances_uri,
                 is_sandbox):
        self.base_uri = base_uri
        self.authorization_uri = authorization_uri
        self.files_uri = files_uri
        self.transfers_uri = transfers_uri
        self.balances_uri = balances_uri
        self.is_sandbox = is_sandbox

    @staticmethod
    def sandbox():
        return Environment(base_uri='https://api.sandbox.checkout.com/',
                           authorization_uri='https://access.sandbox.checkout.com/connect/token',
                           files_uri='https://files.sandbox.checkout.com/',
                           transfers_uri='https://transfers.sandbox.checkout.com/',
                           balances_uri='https://balances.sandbox.checkout.com/',
                           is_sandbox=True)

    @staticmethod
    def production():
        return Environment(base_uri='https://api.checkout.com/',
                           authorization_uri='https://access.checkout.com/connect/token',
                           files_uri='https://files.checkout.com/',
                           transfers_uri='https://transfers.checkout.com/',
                           balances_uri='https://balances.checkout.com/',
                           is_sandbox=False)



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.payments.links.payments_links import PaymentLinkRequest


class PaymentsLinksClient(Client):
    __PAYMENT_LINKS_PATH = 'payment-links'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def get_payment_link(self, payment_link_id: str):
        return self._api_client.get(self.build_path(self.__PAYMENT_LINKS_PATH, payment_link_id),
                                    self._sdk_authorization())

    def create_payment_link(self, payment_link_request: PaymentLinkRequest):
        return self._api_client.post(self.__PAYMENT_LINKS_PATH, self._sdk_authorization(),
                                     payment_link_request)



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.payments.contexts.contexts import PaymentContextsRequest


class PaymentContextsClient(Client):
    __PAYMENT_CONTEXTS_PATH = 'payment-contexts'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY_OR_OAUTH)

    def create_payment_contexts(self, payment_contexts_request: PaymentContextsRequest):
        return self._api_client.post(self.__PAYMENT_CONTEXTS_PATH, self._sdk_authorization(),
                                     payment_contexts_request)

    def get_payment_context_details(self, payment_context_id: str):
        return self._api_client.get(self.build_path(self.__PAYMENT_CONTEXTS_PATH, payment_context_id),
                                    self._sdk_authorization())



from __future__ import absolute_import

from typing import Optional

from requests import Session

from checkout_sdk.environment import Environment
from checkout_sdk.environment_subdomain import EnvironmentSubdomain
from checkout_sdk.sdk_credentials import SdkCredentials


class CheckoutConfiguration:

    def __init__(self,
                 credentials: SdkCredentials,
                 environment: Environment,
                 http_client: Session,
                 environment_subdomain: Optional[EnvironmentSubdomain] = None):
        self.credentials = credentials
        self.environment = environment
        self.http_client = http_client
        self.environment_subdomain = environment_subdomain



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration


class Client:

    def __init__(self, api_client: ApiClient,
                 configuration: CheckoutConfiguration,
                 authorization_type: AuthorizationType):
        self._api_client = api_client
        self._authorization_type = authorization_type
        self._configuration = configuration

    def _sdk_authorization(self, authorization_type: AuthorizationType = None):
        if authorization_type is not None:
            return self._configuration.credentials.get_authorization(authorization_type)
        else:
            return self._configuration.credentials.get_authorization(self._authorization_type)

    def is_sandbox(self):
        return self._configuration.environment.is_sandbox

    @staticmethod
    def build_path(*params):
        return '/'.join(params)



from __future__ import absolute_import

from checkout_sdk.client import Client
from checkout_sdk.files.files import FileRequest


class FilesClient(Client):
    __FILES_PATH = 'files'

    def upload_file(self, file_request: FileRequest):
        return self._api_client.submit_file(self.__FILES_PATH, self._sdk_authorization(), file_request)

    def get_file_details(self, file_id: str):
        return self._api_client.get(self.build_path(self.__FILES_PATH, file_id), self._sdk_authorization())



from datetime import datetime
from enum import Enum

from checkout_sdk.common.common import BankDetails, UpdateCustomerRequest, AccountHolder, Phone
from checkout_sdk.common.enums import AccountType, AccountHolderType, Currency, Country, InstrumentType
from checkout_sdk.payments.payments import PaymentType


# Create
class CreateCustomerInstrumentRequest:
    id: str
    email: str
    name: str
    phone: Phone
    default: bool


class CreateInstrumentRequest:
    type: InstrumentType
    customer: CreateCustomerInstrumentRequest

    def __init__(self, type_p: InstrumentType):
        self.type = type_p


class CreateTokenInstrumentRequest(CreateInstrumentRequest):
    token: str
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(InstrumentType.TOKEN)


class InstrumentData:
    account_number: str
    country: Country
    currency: Currency
    payment_type: PaymentType
    mandate_id: str
    date_of_signature: datetime


class CreateSepaInstrumentRequest(CreateInstrumentRequest):
    token: str
    instrument_data: InstrumentData
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(InstrumentType.SEPA)


class CreateBankAccountInstrumentRequest(CreateInstrumentRequest):
    account_type: AccountType
    account_number: str
    bank_code: str
    branch_code: str
    iban: str
    bban: str
    swift_bic: str
    currency: Currency
    processing_channel_id: str
    account_holder: AccountHolder
    bank_details: BankDetails
    bank: BankDetails

    def __init__(self):
        super().__init__(InstrumentType.BANK_ACCOUNT)


# Update
class UpdateInstrumentRequest:
    type: InstrumentType

    def __init__(self, type_p: InstrumentType):
        self.type = type_p


class UpdateTokenInstrumentRequest(UpdateInstrumentRequest):
    token: str

    def __init__(self):
        super().__init__(InstrumentType.TOKEN)


class UpdateCardInstrumentRequest(UpdateInstrumentRequest):
    expiry_month: int
    expiry_year: int
    name: str
    customer: UpdateCustomerRequest
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(InstrumentType.CARD)


class UpdateBankAccountInstrumentRequest(UpdateInstrumentRequest):
    account_type: AccountType
    account_number: str
    bank_code: str
    branch_code: str
    iban: str
    bban: str
    swift_bic: str
    currency: Currency
    country: Country
    processing_channel_id: str
    account_holder: AccountHolder
    bank_details: BankDetails
    customer: UpdateCustomerRequest

    def __init__(self):
        super().__init__(InstrumentType.BANK_ACCOUNT)


class PaymentNetwork(str, Enum):
    LOCAL = 'local'
    SEPA = 'sepa'
    FPS = 'Fps'
    ACH = 'Ach'
    FEDWIRE = 'Fedwire'
    SWIFT = 'Swift'


class BankAccountFieldQuery:
    account_holder_type: AccountHolderType
    payment_network: PaymentNetwork



from __future__ import absolute_import

import re
from abc import ABCMeta

from checkout_sdk.checkout_sdk_builder import CheckoutSdkBuilder
from checkout_sdk.exception import CheckoutArgumentException


def validate_key(regex, key):
    return re.match(regex, key)


def validate_secret_key(regex, secret_key=None):
    if secret_key is None or not validate_key(regex, secret_key):
        raise CheckoutArgumentException('invalid secret key')


def validate_public_key(regex, public_key=None):
    if not public_key:
        return
    if validate_key(regex, public_key):
        return
    raise CheckoutArgumentException('invalid public key')


class StaticKeysBuilder(CheckoutSdkBuilder, metaclass=ABCMeta):
    _public_key: str = None
    _secret_key: str = None

    def __init__(self):
        super().__init__()

    def secret_key(self, secret_key):
        self._secret_key = secret_key
        return self

    def public_key(self, public_key):
        self._public_key = public_key
        return self



from enum import Enum


class WorkflowConditionType(str, Enum):
    EVENT = 'event'
    ENTITY = 'entity'
    PROCESSING_CHANNEL = 'processing_channel'


class WorkflowActionType(str, Enum):
    WEBHOOK = 'webhook'


class WorkflowConditionRequest:
    type: WorkflowConditionType

    def __init__(self, type_p: WorkflowConditionType):
        self.type = type_p


class EventWorkflowConditionRequest(WorkflowConditionRequest):
    events: dict

    def __init__(self):
        super().__init__(WorkflowConditionType.EVENT)


class EntityWorkflowConditionRequest(WorkflowConditionRequest):
    entities: list

    def __init__(self):
        super().__init__(WorkflowConditionType.ENTITY)


class ProcessingChannelWorkflowConditionRequest(WorkflowConditionRequest):
    processing_channels: list

    def __init__(self):
        super().__init__(WorkflowConditionType.PROCESSING_CHANNEL)


class WorkflowActionRequest:
    type: WorkflowActionType

    def __init__(self, type_p: WorkflowActionType):
        self.type = type_p


class WebhookSignature:
    method: str
    key: str


class WebhookWorkflowActionRequest(WorkflowActionRequest):
    url: str
    headers: dict
    signature: WebhookSignature

    def __init__(self):
        super().__init__(WorkflowActionType.WEBHOOK)


class CreateWorkflowRequest:
    name: str
    active: bool
    conditions: list  # WorkflowConditionRequest
    actions: list  # WorkflowActionRequest


class UpdateWorkflowRequest:
    name: str
    active: bool
    conditions: list  # WorkflowConditionRequest
    actions: list  # WorkflowActionRequest


class ReflowRequest:
    workflows: list


class ReflowByEventsRequest(ReflowRequest):
    events: list


class ReflowBySubjectsRequest(ReflowRequest):
    subjects: list


class EventTypesRequest:
    event_types: list



from enum import Enum

from checkout_sdk.common.common import Phone, Address
from checkout_sdk.common.common import ResidentialStatusType, AccountHolderIdentification
from checkout_sdk.common.enums import Currency, InstrumentType, Country, AccountType, AccountHolderType, DocumentType


class ScheduleFrequency(str, Enum):
    WEEKLY = 'weekly'
    DAILY = 'daily'
    MONTHLY = 'monthly'


class DaySchedule(str, Enum):
    MONDAY = 'monday'
    TUESDAY = 'tuesday'
    WEDNESDAY = 'wednesday'
    THURSDAY = 'thursday'
    FRIDAY = 'friday'
    SATURDAY = 'saturday'
    SUNDAY = 'sunday'


class BusinessType(str, Enum):
    GENERAL_PARTNERSHIP = 'general_partnership'
    LIMITED_PARTNERSHIP = 'limited_partnership'
    PUBLIC_LIMITED_COMPANY = 'public_limited_company'
    LIMITED_COMPANY = 'limited_company'
    PROFESSIONAL_ASSOCIATION = 'professional_association'
    UNINCORPORATED_ASSOCIATION = 'unincorporated_association'
    AUTO_ENTREPRENEUR = 'auto_entrepreneur'


class EntityRoles(str, Enum):
    UBO = 'ubo'
    LEGAL_REPRESENTATIVE = 'legal_representative'


class EntityEmailAddresses:
    primary: str


class Invitee:
    email: str


class ContactDetails:
    phone: Phone
    email_addresses: EntityEmailAddresses
    invitee: Invitee


class Profile:
    urls: list
    mccs: list
    default_holding_currency: Currency
    holding_currencies: list


class EntityDocument:
    file_id: str
    type: str


class EntityIdentificationDocument:
    type: DocumentType
    front: str
    back: str


class EntityIdentification:
    national_id_number: str
    document: EntityIdentificationDocument


class DateOfBirth:
    day: int
    month: int
    year: int


class PlaceOfBirth:
    country: Country


class CompanyVerificationType(str, Enum):
    INCORPORATION_DOCUMENT = 'incorporation_document'
    ARTICLES_OF_ASSOCIATION = 'articles_of_association'


class CompanyVerification:
    type: CompanyVerificationType
    front: str


class TaxVerificationType(str, Enum):
    EIN_LETTER = 'ein_letter'


class TaxVerification:
    type: TaxVerificationType
    front: str


class ArticlesOfAssociationType(str, Enum):
    MEMORANDUM_OF_ASSOCIATION = "memorandum_of_association"
    ARTICLES_OF_ASSOCIATION = "articles_of_association"


class BankVerificationType(str, Enum):
    BANK_STATEMENT = 'bank_statement'


class BankVerification:
    type: BankVerificationType
    front: str


class ShareholderStructureType(str, Enum):
    CERTIFIED_SHAREHOLDER_STRUCTURE = 'certified_shareholder_structure'


class ShareholderStructure:
    type: ShareholderStructureType
    front: str


class ProofOfLegalityType(str, Enum):
    PROOF_OF_LEGALITY = 'proof_of_legality'


class ProofOfLegality:
    type: ProofOfLegalityType
    front: str


class ProofOfPrincipalAddressType(str, Enum):
    PROOF_OF_ADDRESS = 'proof_of_address'


class ProofOfPrincipalAddress:
    type: ProofOfPrincipalAddressType
    front: str


class AdditionalDocument:
    front: str


class FinancialVerificationType(str, Enum):
    FINANCIAL_STATEMENT = 'financial_statement'


class FinancialVerification:
    type: FinancialVerificationType
    front: str


class OnboardSubEntityDocuments:
    identity_verification: EntityIdentificationDocument
    company_verification: CompanyVerification
    articles_of_association: ArticlesOfAssociationType
    bank_verification: BankVerification
    shareholder_structure: ShareholderStructure
    proof_of_legality: ProofOfLegality
    proof_of_principal_address: ProofOfPrincipalAddress
    additional_document1: AdditionalDocument
    additional_document2: AdditionalDocument
    additional_document3: AdditionalDocument
    tax_verification: TaxVerification
    financial_verification: FinancialVerification


class EntityRepresentative:
    first_name: str
    middle_name: str
    last_name: str
    address: Address
    identification: EntityIdentification
    phone: Phone
    date_of_birth: DateOfBirth
    place_of_birth: PlaceOfBirth
    roles: list  # accounts.EntityRoles
    documents: OnboardSubEntityDocuments


class EntityFinancialDocuments:
    bank_statement: EntityDocument
    financial_statement: EntityDocument


class EntityFinancialDetails:
    annual_processing_volume: int
    average_transaction_value: int
    highest_transaction_value: int
    documents: EntityFinancialDocuments
    currency: Currency


class DateOfIncorporation:
    month: int
    year: int


class Company:
    business_registration_number: str
    business_type: BusinessType
    legal_name: str
    trading_name: str
    date_of_incorporation: DateOfIncorporation
    regulatory_licence_number: str
    principal_address: Address
    registered_address: Address
    representatives: list  # EntityRepresentative
    document: EntityDocument
    financial_details: EntityFinancialDetails


class Identification:
    national_id_number: str
    document: EntityIdentificationDocument


class Individual:
    first_name: str
    middle_name: str
    last_name: str
    trading_name: str
    national_tax_id: str
    registered_address: Address
    date_of_birth: DateOfBirth
    place_of_birth: PlaceOfBirth
    identification: Identification
    financial_details: EntityFinancialDetails


class ProcessingDetails:
    settlement_country: str
    target_countries: list  # str
    annual_processing_volume: int
    average_transaction_value: int
    highest_transaction_value: int
    currency: Currency


class AdditionalInfo:
    field1: str
    field2: str
    field3: str


class OnboardEntityRequest:
    reference: str
    is_draft: bool
    profile: Profile
    contact_details: ContactDetails
    company: Company
    processing_details: ProcessingDetails
    individual: Individual
    documents: OnboardSubEntityDocuments
    additional_info: AdditionalInfo


class InstrumentDocument:
    type: str
    file_id: str


class InstrumentDetails:
    pass


class InstrumentDetailsFasterPayments(InstrumentDetails):
    account_number: str
    bank_code: str


class InstrumentDetailsSepa(InstrumentDetails):
    iban: str
    swift_bic: str


class InstrumentDetailsCardToken(InstrumentDetails):
    token: str


class BankDetails:
    name: str
    branch: str
    address: Address


class AccountsAccountHolder:
    type: AccountHolderType
    tax_id: str
    date_of_birth: DateOfBirth
    country_of_birth: Country
    residential_status: ResidentialStatusType
    billing_address: Address
    phone: Phone
    identification: AccountHolderIdentification
    email: str


class AccountsCorporateAccountHolder(AccountsAccountHolder):
    company_name: str


class AccountsIndividualAccountHolder(AccountsAccountHolder):
    first_name: str
    last_name: str


class AccountsPaymentInstrument:
    type = InstrumentType.BANK_ACCOUNT
    label: str
    account_type: AccountType
    account_number: str
    bank_code: str
    branch_code: str
    iban: str
    bban: str
    swift_bic: str
    currency: Currency
    country: Country
    document: InstrumentDocument
    account_holder: AccountsAccountHolder
    bank: BankDetails


class PaymentInstrumentRequest:
    label: str
    type = InstrumentType
    currency: Currency
    country: Country
    default: bool
    document: InstrumentDocument
    instrument_details: InstrumentDetails


class Headers:
    if_match: str


class UpdatePaymentInstrumentRequest:
    label: str
    default: bool
    headers: Headers


class ScheduleRequest:
    frequency: ScheduleFrequency

    def __init__(self, frequency_p: ScheduleFrequency):
        self.frequency = frequency_p


class ScheduleFrequencyDailyRequest(ScheduleRequest):
    def __init__(self):
        super().__init__(ScheduleFrequency.DAILY)


class ScheduleFrequencyMonthlyRequest(ScheduleRequest):
    by_month_day: list  # int

    def __init__(self):
        super().__init__(ScheduleFrequency.MONTHLY)


class ScheduleFrequencyWeeklyRequest(ScheduleRequest):
    by_day: list  # DaySchedule

    def __init__(self):
        super().__init__(ScheduleFrequency.WEEKLY)


class UpdateScheduleRequest:
    enabled: bool
    threshold: int
    recurrence: ScheduleRequest


class PaymentInstrumentsQuery:
    status: str



from enum import Enum

from checkout_sdk.common.enums import Currency


class TransferType(str, Enum):
    COMMISSION = 'commission'
    PROMOTION = 'promotion'
    REFUND = 'refund'


class TransferSource:
    id: str
    amount: int
    currency: Currency


class TransferDestination:
    id: str


class CreateTransferRequest:
    reference: str
    transfer_type: TransferType
    source: TransferSource
    destination: TransferDestination



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.events.events import RetrieveEventsRequest


class EventsClient(Client):
    __EVENTS_PATH = 'events'
    __NOTIFICATIONS_PATH = 'notifications'
    __WEBHOOKS_PATH = 'webhooks'
    __RETRY_PATH = 'retry'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def retrieve_all_event_types(self, version: str = None):
        event_types_path = "event-types"
        if version is not None:
            event_types_path += "?version=" + version
        return self._api_client.get(event_types_path, self._sdk_authorization())

    def retrieve_events(self, events_request: RetrieveEventsRequest):
        return self._api_client.get(self.__EVENTS_PATH, self._sdk_authorization(), events_request)

    def retrieve_event(self, event_id: str):
        return self._api_client.get(self.build_path(self.__EVENTS_PATH, event_id), self._sdk_authorization())

    def retrieve_event_notification(self, event_id: str, notification_id: str):
        return self._api_client.get(
            self.build_path(self.__EVENTS_PATH, event_id, self.__NOTIFICATIONS_PATH, notification_id),
            self._sdk_authorization())

    def retry_webhook(self, event_id: str, webhook_id: str):
        return self._api_client.post(
            self.build_path(self.__EVENTS_PATH, event_id, self.__WEBHOOKS_PATH, webhook_id, self.__RETRY_PATH),
            self._sdk_authorization())

    def retry_all_webhooks(self, event_id: str):
        return self._api_client.post(
            self.build_path(self.__EVENTS_PATH, event_id, self.__WEBHOOKS_PATH, self.__RETRY_PATH),
            self._sdk_authorization())



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.reports.reports import ReportsQuery


class ReportsClient(Client):
    __REPORTS_PATH = 'reports'
    __FILES_PATH = 'files'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY_OR_OAUTH)

    def get_all_reports(self, query: ReportsQuery):
        return self._api_client.get(self.__REPORTS_PATH, self._sdk_authorization(), query)

    def get_report_details(self, report_id: str):
        return self._api_client.get(self.build_path(self.__REPORTS_PATH, report_id), self._sdk_authorization())

    def get_report_file(self, report_id: str, file_id: str):
        return self._api_client.get(self.build_path(self.__REPORTS_PATH, report_id, self.__FILES_PATH, file_id),
                                    self._sdk_authorization())



from enum import Enum

from checkout_sdk.common.common import Phone, Address
from checkout_sdk.common.enums import DocumentType


class CardholderType(str, Enum):
    INDIVIDUAL = 'individual'


class CardholderDocument:
    type: DocumentType
    front_document_id: str
    back_document_id: str


class CardholderRequest:
    type: CardholderType
    reference: str
    entity_id: str
    first_name: str
    middle_name: str
    last_name: str
    email: str
    phone_number: Phone
    date_of_birth: str
    billing_address: Address
    residency_address: Address
    document: CardholderDocument



from __future__ import absolute_import

from datetime import datetime


class OAuthAccessToken:
    token: str
    token_type: str
    expiration_date: datetime

    def __init__(self, token: str, token_type: str, expiration_date: datetime):
        self.token = token
        self.token_type = token_type
        self.expiration_date = expiration_date

    def is_valid(self):
        if self.token is None:
            return False
        return self.expiration_date > datetime.now()



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.instruments.instruments_previous import UpdateInstrumentRequest, CreateInstrumentRequest


class InstrumentsClient(Client):
    __INSTRUMENTS = 'instruments'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def get(self, instrument_id: str):
        return self._api_client.get(self.build_path(self.__INSTRUMENTS, instrument_id), self._sdk_authorization())

    def create(self, create_instrument_request: CreateInstrumentRequest):
        return self._api_client.post(self.__INSTRUMENTS, self._sdk_authorization(), create_instrument_request)

    def update(self, instrument_id: str, update_instrument_request: UpdateInstrumentRequest):
        return self._api_client.patch(
            self.build_path(self.__INSTRUMENTS, instrument_id),
            self._sdk_authorization(),
            update_instrument_request)

    def delete(self, instrument_id: str):
        return self._api_client.delete(self.build_path(self.__INSTRUMENTS, instrument_id), self._sdk_authorization())



from datetime import datetime

from checkout_sdk.common.common import CustomerRequest, CustomerRetry
from checkout_sdk.common.enums import Currency
from checkout_sdk.payments.payments import BillingDescriptor, PaymentType, ShippingDetails, ThreeDsRequest, \
    RiskRequest, PaymentRecipient, ProcessingSettings, PaymentSender
from checkout_sdk.payments.payments_previous import BillingInformation


class HostedPaymentsSessionRequest:
    amount: int
    currency: Currency
    payment_type: PaymentType.REGULAR
    payment_ip: str
    billing_descriptor: BillingDescriptor
    reference: str
    description: str
    display_name: str
    processing_channel_id: str
    amount_allocations: list  # values of AmountAllocations
    customer: CustomerRequest
    shipping: ShippingDetails
    billing: BillingInformation
    recipient: PaymentRecipient
    processing: ProcessingSettings
    allow_payment_methods: list  # PaymentSourceType
    disabled_payment_methods: list  # PaymentSourceType
    products: list  # common.Product
    risk: RiskRequest
    customer_retry: CustomerRetry
    sender: PaymentSender
    success_url: str
    cancel_url: str
    failure_url: str
    metadata: dict
    locale: str
    three_ds: ThreeDsRequest
    capture: bool
    capture_on: datetime



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.balances.balances import BalancesQuery
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client


class BalancesClient(Client):
    __BALANCES_PATH = 'balances'

    def __init__(self, api_client: ApiClient,
                 configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY_OR_OAUTH)

    def retrieve_entity_balances(self, entity_id: str, balances_query: BalancesQuery):
        return self._api_client.get(self.build_path(self.__BALANCES_PATH, entity_id), self._sdk_authorization(),
                                    balances_query)



from enum import Enum


class SourceType(str, Enum):
    ID = 'id'
    TOKEN = 'token'


class AbstractSource():
    type: SourceType

    def __init__(self, type_p: SourceType):
        self.type = type_p


class IdSource(AbstractSource):
    id: str
    cvv_token: str

    def __init__(self):
        super().__init__(SourceType.ID)


class TokenSource(AbstractSource):
    token: str

    def __init__(self):
        super().__init__(SourceType.TOKEN)


class SignatureType(str, Enum):
    DLOCAL = 'dlocal'


class AbstractSignature():
    type: SignatureType

    def __init__(self, type_p: SignatureType):
        self.type = type_p


class DlocalParameters:
    secret_key: str


class DlocalSignature(AbstractSignature):
    dlocal_parameters: DlocalParameters

    def __init__(self):
        super().__init__(SignatureType.DLOCAL)


class MethodType(str, Enum):
    GET = 'GET'
    POST = 'POST'
    PUT = 'PUT'
    DELETE = 'DELETE'
    PATCH = 'PATCH'
    HEAD = 'HEAD'
    OPTIONS = 'OPTIONS'
    TRACE = 'TRACE'


class Headers:
    raw: dict
    encrypted: str = None


class DestinationRequest:
    url: str
    method: MethodType
    headers: Headers
    body: str
    signature: DlocalSignature = None


class NetworkToken:
    enabled: bool = None
    request_cryptogram: bool = None


class ForwardRequest:
    source: AbstractSource
    destination_request: DestinationRequest
    reference: str = None
    processing_channel_id: str = None
    network_token: NetworkToken = None



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.tokens.tokens import WalletTokenRequest, CardTokenRequest


class TokensClient(Client):
    __TOKENS = 'tokens'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.PUBLIC_KEY)

    def request_card_token(self, request: CardTokenRequest):
        return self._api_client.post(self.__TOKENS, self._sdk_authorization(), request)

    def request_wallet_token(self, request: WalletTokenRequest):
        return self._api_client.post(self.__TOKENS, self._sdk_authorization(), request)



from __future__ import absolute_import

from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.exception import CheckoutAuthorizationException
from checkout_sdk.platform_type import PlatformType
from checkout_sdk.sdk_authorization import SdkAuthorization
from checkout_sdk.sdk_credentials import SdkCredentials


class SessionSecretSdkCredentials(SdkCredentials):

    def __init__(self, secret):
        self.secret = secret

    def get_authorization(self, authorization_type: AuthorizationType):
        if AuthorizationType.CUSTOM == authorization_type:
            if self.secret is None:
                raise CheckoutAuthorizationException.invalid_key(AuthorizationType.CUSTOM)
            return SdkAuthorization(PlatformType.CUSTOM, self.secret)
        raise CheckoutAuthorizationException.invalid_authorization(authorization_type=authorization_type)



from __future__ import absolute_import

import requests

from checkout_sdk.http_client_interface import HttpClientBuilderInterface


class DefaultHttpClientBuilder(HttpClientBuilderInterface):

    def __init__(self):
        self.session = requests.Session()

    def get_client(self):
        return self.session



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.transfers.transfers import CreateTransferRequest


class TransfersClient(Client):
    __TRANSFERS_PATH = 'transfers'

    def __init__(self, api_client: ApiClient,
                 configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY_OR_OAUTH)

    def initiate_transfer_of_funds(self, create_transfer_request: CreateTransferRequest, idempotency_key: str):
        return self._api_client.post(self.__TRANSFERS_PATH, self._sdk_authorization(), create_transfer_request,
                                     idempotency_key)

    def retrieve_a_transfer(self, transfer_id):
        return self._api_client.get(self.build_path(self.__TRANSFERS_PATH, transfer_id),
                                    self._sdk_authorization())



from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.forward.forward import ForwardRequest


class ForwardClient(Client):
    __FORWARD_PATH = 'forward'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY_OR_OAUTH)

    def forward_request(self, request: ForwardRequest):
        return self._api_client.post(self.__FORWARD_PATH, self._sdk_authorization(), request)

    def get(self, request_id: str):
        return self._api_client.get(self.build_path(self.__FORWARD_PATH, request_id), self._sdk_authorization())



from __future__ import absolute_import

import json
import logging
import mimetypes
from pathlib import Path

from requests.exceptions import HTTPError

from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.checkout_response import ResponseWrapper
from checkout_sdk.exception import CheckoutApiException, CheckoutException
from checkout_sdk.files.files import FileRequest
from checkout_sdk.json_serializer import JsonSerializer
from checkout_sdk.properties import VERSION
from checkout_sdk.sdk_authorization import SdkAuthorization
from checkout_sdk.utils import map_to_http_metadata


def get_file_request(file_request: FileRequest, multipart_file=None):
    key_file = 'file' if multipart_file is None else multipart_file
    file_name = Path(file_request.file).name
    mime_type = mimetypes.MimeTypes().guess_type(file_name)[0]
    files = {key_file: (file_name, open(file_request.file, 'rb'), mime_type)}
    json_body = [('purpose', file_request.purpose)]
    return files, json_body


class ApiClient:
    _logger = logging.getLogger('checkout')

    def __init__(self, configuration: CheckoutConfiguration, base_uri: str):
        self._http_client = configuration.http_client
        self._base_uri = base_uri

    def get(self,
            path,
            authorization: SdkAuthorization,
            params=None):
        return self.invoke(method='GET', path=path, authorization=authorization, params=params)

    def post(self,
             path,
             authorization: SdkAuthorization,
             request=None,
             idempotency_key: str = None):
        return self.invoke(method='POST', path=path, authorization=authorization, body=request,
                           idempotency_key=idempotency_key)

    def put(self,
            path,
            authorization: SdkAuthorization,
            request=None):
        return self.invoke(method='PUT', path=path, authorization=authorization, body=request)

    def patch(self,
              path,
              authorization: SdkAuthorization,
              request=None):
        return self.invoke(method='PATCH', path=path, authorization=authorization, body=request)

    def delete(self,
               path,
               authorization: SdkAuthorization):
        return self.invoke(method='DELETE', path=path, authorization=authorization)

    def submit_file(self,
                    path,
                    authorization: SdkAuthorization,
                    file_request: FileRequest,
                    multipart_file=None):
        return self.invoke(method='POST', path=path, authorization=authorization, file_request=file_request,
                           multipart_file=multipart_file)

    def invoke(self,
               method: str,
               path: str,
               authorization: SdkAuthorization,
               body=None,
               idempotency_key: str = None,
               params=None,
               file_request: FileRequest = None,
               multipart_file=None):

        headers = {
            'User-Agent': 'checkout-sdk-python/' + VERSION,
            'Accept': 'application/json',
            'Authorization': authorization.get_authorization_header(),
            'Content-Type': 'application/json'}

        if idempotency_key is not None:
            headers['Cko-Idempotency-Key'] = idempotency_key

        base_uri = self._base_uri + path

        try:
            json_body = None
            params_dict = None
            files = None

            if body is not None:
                json_body = json.dumps(body, cls=JsonSerializer)
            elif params is not None:
                params_dict = json.loads(json.dumps(params, cls=JsonSerializer))
            elif file_request is not None:
                headers.pop('Content-Type')
                files, json_body = get_file_request(file_request, multipart_file)

            self._logger.info(method + ' ' + path)

            response = self._http_client.request(method=method,
                                                 url=base_uri,
                                                 headers=headers,
                                                 params=params_dict,
                                                 data=json_body,
                                                 files=files)

            response.raise_for_status()
        except HTTPError as err:
            self._logger.error(err)
            raise CheckoutApiException(err.response) from err
        except OSError as err:
            error = err.strerror
            raise CheckoutException(error) from err

        http_metadata = map_to_http_metadata(response)
        if response.text:
            if response.headers.get('Content-Type').startswith('application/json'):
                contents = response.json()
            else:
                contents = response.text
            return ResponseWrapper(http_metadata, contents)
        else:
            return ResponseWrapper(http_metadata)



import re
from urllib.parse import urlparse, urlunparse

from checkout_sdk.environment import Environment


class EnvironmentSubdomain:
    def __init__(self, environment: Environment, subdomain: str):
        self.base_uri = self.add_subdomain_to_api_url_environment(environment, subdomain)

    @staticmethod
    def add_subdomain_to_api_url_environment(environment: Environment, subdomain: str):
        api_url = environment.base_uri
        new_environment = api_url

        regex = r'^[0-9a-z]+$'
        if re.match(regex, subdomain):
            url_parts = urlparse(api_url)
            if url_parts.port:
                new_host = subdomain + '.' + url_parts.hostname + ':' + url_parts.port
            else:
                new_host = subdomain + '.' + url_parts.hostname

            new_url_parts = (
                url_parts.scheme,
                new_host,
                url_parts.path,
                url_parts.params,
                url_parts.query,
                url_parts.fragment
            )

            new_environment = urlunparse(new_url_parts)

        return new_environment

    def base_uri(self) -> str:
        return self.base_uri



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.issuing.cardholders import CardholderRequest
from checkout_sdk.issuing.cards import CardRequest, ThreeDsEnrollmentRequest, UpdateThreeDsEnrollmentRequest, \
    CardCredentialsQuery, RevokeRequest, SuspendRequest
from checkout_sdk.issuing.controls import CardControlRequest, CardControlsQuery, UpdateCardControlRequest
from checkout_sdk.issuing.testing import CardAuthorizationRequest, SimulationRequest


class IssuingClient(Client):
    __ISSUING = 'issuing'
    __CARDHOLDERS = 'cardholders'
    __CARDS = 'cards'
    __THREE_DS = '3ds-enrollment'
    __ACTIVATE = 'activate'
    __CREDENTIALS = 'credentials'
    __REVOKE = 'revoke'
    __SUSPEND = 'suspend'
    __CONTROLS = 'controls'
    __SIMULATE = 'simulate'
    __AUTHORIZATIONS = 'authorizations'
    __PRESENTMENTS = 'presentments'
    __REVERSALS = 'reversals'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY_OR_OAUTH)

    def create_cardholder(self, cardholder_request: CardholderRequest):
        return self._api_client.post(self.build_path(self.__ISSUING, self.__CARDHOLDERS),
                                     self._sdk_authorization(),
                                     cardholder_request)

    def get_cardholder(self, cardholder_id: str):
        return self._api_client.get(self.build_path(self.__ISSUING, self.__CARDHOLDERS, cardholder_id),
                                    self._sdk_authorization())

    def get_cardholder_cards(self, cardholder_id: str):
        return self._api_client.get(self.build_path(self.__ISSUING, self.__CARDHOLDERS, cardholder_id, self.__CARDS),
                                    self._sdk_authorization())

    def create_card(self, card_request: CardRequest):
        return self._api_client.post(self.build_path(self.__ISSUING, self.__CARDS),
                                     self._sdk_authorization(),
                                     card_request)

    def get_card_details(self, card_id: str):
        return self._api_client.get(self.build_path(self.__ISSUING, self.__CARDS, card_id),
                                    self._sdk_authorization())

    def enroll_three_ds(self, card_id: str, three_ds_enrollment_request: ThreeDsEnrollmentRequest):
        return self._api_client.post(self.build_path(self.__ISSUING, self.__CARDS, card_id, self.__THREE_DS),
                                     self._sdk_authorization(),
                                     three_ds_enrollment_request)

    def update_three_ds_enrollment(self, card_id: str, update_three_ds_enrollment: UpdateThreeDsEnrollmentRequest):
        return self._api_client.patch(self.build_path(self.__ISSUING, self.__CARDS, card_id, self.__THREE_DS),
                                      self._sdk_authorization(),
                                      update_three_ds_enrollment)

    def get_three_ds_details(self, card_id: str):
        return self._api_client.get(self.build_path(self.__ISSUING, self.__CARDS, card_id, self.__THREE_DS),
                                    self._sdk_authorization())

    def activate_card(self, card_id: str):
        return self._api_client.post(self.build_path(self.__ISSUING, self.__CARDS, card_id, self.__ACTIVATE),
                                     self._sdk_authorization())

    def get_card_credentials(self, card_id: str, card_credentials_query: CardCredentialsQuery):
        return self._api_client.get(self.build_path(self.__ISSUING, self.__CARDS, card_id, self.__CREDENTIALS),
                                    self._sdk_authorization(),
                                    card_credentials_query)

    def revoke_card(self, card_id: str, revoke_request: RevokeRequest):
        return self._api_client.post(self.build_path(self.__ISSUING, self.__CARDS, card_id, self.__REVOKE),
                                     self._sdk_authorization(),
                                     revoke_request)

    def suspend_card(self, card_id: str, suspend_request: SuspendRequest):
        return self._api_client.post(self.build_path(self.__ISSUING, self.__CARDS, card_id, self.__SUSPEND),
                                     self._sdk_authorization(),
                                     suspend_request)

    def create_control(self, control_request: CardControlRequest):
        return self._api_client.post(self.build_path(self.__ISSUING, self.__CONTROLS),
                                     self._sdk_authorization(),
                                     control_request)

    def get_card_controls(self, query: CardControlsQuery):
        return self._api_client.get(self.build_path(self.__ISSUING, self.__CONTROLS),
                                    self._sdk_authorization(),
                                    query)

    def get_card_control_details(self, control_id: str):
        return self._api_client.get(self.build_path(self.__ISSUING, self.__CONTROLS, control_id),
                                    self._sdk_authorization())

    def update_card_control(self, control_id: str, update_request: UpdateCardControlRequest):
        return self._api_client.put(self.build_path(self.__ISSUING, self.__CONTROLS, control_id),
                                    self._sdk_authorization(),
                                    update_request)

    def remove_control(self, control_id: str):
        return self._api_client.delete(self.build_path(self.__ISSUING, self.__CONTROLS, control_id),
                                       self._sdk_authorization())

    def simulate_authorization(self, authorization_request: CardAuthorizationRequest):
        return self._api_client.post(self.build_path(self.__ISSUING, self.__SIMULATE, self.__AUTHORIZATIONS),
                                     self._sdk_authorization(),
                                     authorization_request)

    def simulate_increment(self, transaction_id: str, increment_request: SimulationRequest):
        return self._api_client.post(
            self.build_path(
                self.__ISSUING, self.__SIMULATE, self.__AUTHORIZATIONS, transaction_id, self.__AUTHORIZATIONS),
            self._sdk_authorization(),
            increment_request)

    def simulate_clearing(self, transaction_id: str, clearing_request: SimulationRequest):
        return self._api_client.post(
            self.build_path(
                self.__ISSUING, self.__SIMULATE, self.__AUTHORIZATIONS, transaction_id, self.__PRESENTMENTS),
            self._sdk_authorization(),
            clearing_request)

    def simulate_reversal(self, transaction_id: str, reversal_request: SimulationRequest):
        return self._api_client.post(
            self.build_path(
                self.__ISSUING, self.__SIMULATE, self.__AUTHORIZATIONS, transaction_id, self.__REVERSALS),
            self._sdk_authorization(),
            reversal_request)



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.payments.payments_previous import PaymentRequest, PayoutRequest, CaptureRequest
from checkout_sdk.payments.payments import RefundRequest, VoidRequest, PaymentsQueryFilter


class PaymentsClient(Client):
    __PAYMENTS_PATH = 'payments'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def request_payment(self, payment_request: PaymentRequest, idempotency_key: str = None):
        return self._api_client.post(self.__PAYMENTS_PATH,
                                     self._sdk_authorization(),
                                     payment_request,
                                     idempotency_key)

    def request_payout(self, payout_request: PayoutRequest, idempotency_key: str = None):
        return self._api_client.post(self.__PAYMENTS_PATH,
                                     self._sdk_authorization(),
                                     payout_request,
                                     idempotency_key)

    def get_payments_list(self, query: PaymentsQueryFilter):
        return self._api_client.get(self.__PAYMENTS_PATH, self._sdk_authorization(), query)

    def get_payment_details(self, payment_id: str):
        return self._api_client.get(self.build_path(self.__PAYMENTS_PATH, payment_id),
                                    self._sdk_authorization())

    def get_payment_actions(self, payment_id: str):
        return self._api_client.get(self.build_path(self.__PAYMENTS_PATH, payment_id, 'actions'),
                                    self._sdk_authorization())

    def capture_payment(self, payment_id: str, capture_request: CaptureRequest = None, idempotency_key: str = None):
        return self._api_client.post(self.build_path(self.__PAYMENTS_PATH, payment_id, 'captures'),
                                     self._sdk_authorization(),
                                     capture_request,
                                     idempotency_key)

    def refund_payment(self, payment_id: str, refund_request: RefundRequest = None, idempotency_key: str = None):
        return self._api_client.post(self.build_path(self.__PAYMENTS_PATH, payment_id, 'refunds'),
                                     self._sdk_authorization(),
                                     refund_request,
                                     idempotency_key)

    def void_payment(self, payment_id: str, void_request: VoidRequest = None, idempotency_key: str = None):
        return self._api_client.post(self.build_path(self.__PAYMENTS_PATH, payment_id, 'voids'),
                                     self._sdk_authorization(),
                                     void_request,
                                     idempotency_key)



from __future__ import absolute_import

from datetime import datetime
from enum import Enum

from checkout_sdk.common.enums import Country, PaymentSourceType
from checkout_sdk.payments.payments import Payer, PaymentMethodDetails
from checkout_sdk.payments.payments_previous import RequestSource
from checkout_sdk.tokens.tokens import ApplePayTokenData


class IntegrationType(str, Enum):
    DIRECT = 'direct'
    REDIRECT = 'redirect'


class KlarnaCustomer:
    date_of_birth: str
    gender: str


class KlarnaProduct:
    name: str
    quantity: int
    unit_price: int
    tax_rate: int
    total_amount: int
    total_tax_amount: int


class GiroPayInfoFields:
    label: str
    text: str


class RequestAlipaySource(RequestSource):
    def __init__(self):
        super().__init__(PaymentSourceType.ALIPAY)


class RequestBancontactSource(RequestSource):
    payment_country: Country
    account_holder_name: str
    billing_descriptor: str
    language: str

    def __init__(self):
        super().__init__(PaymentSourceType.BANCONTACT)


class RequestBenefitPaySource(RequestSource):
    integration_type: str

    def __init__(self):
        super().__init__(PaymentSourceType.BENEFITPAY)


class RequestBoletoSource(RequestSource):
    integration_type: IntegrationType = IntegrationType.REDIRECT
    country: Country
    description: str
    payer: Payer

    def __init__(self):
        super().__init__(PaymentSourceType.BOLETO)


class RequestEpsSource(RequestSource):
    purpose: str
    bic: str

    def __init__(self):
        super().__init__(PaymentSourceType.EPS)


class RequestFawrySource(RequestSource):
    description: str
    customer_profile_id: str
    customer_mobile: str
    customer_email: str
    expires_on: datetime
    products: list  # FawryProduct

    def __init__(self):
        super().__init__(PaymentSourceType.FAWRY)


class RequestGiropaySource(RequestSource):
    purpose: str
    bic: str
    info_fields: list  # GiroPayInfoFields

    def __init__(self):
        super().__init__(PaymentSourceType.GIROPAY)


class RequestIdealSource(RequestSource):
    bic: str
    description: str
    language: str

    def __init__(self):
        super().__init__(PaymentSourceType.IDEAL)


class RequestKlarnaSource(RequestSource):
    authorization_token: str
    locale: str
    purchase_country: Country
    auto_capture: bool
    billing_address: dict
    shipping_address: dict
    tax_amount: int
    products: list  # KlarnaProduct
    customer: KlarnaCustomer
    merchant_reference1: str
    merchant_reference2: str
    merchant_data: str
    attachment: dict
    custom_payment_method_ids: list

    def __init__(self):
        super().__init__(PaymentSourceType.KLARNA)


class RequestKnetSource(RequestSource):
    language: str
    user_defined_field1: str
    user_defined_field2: str
    user_defined_field3: str
    user_defined_field4: str
    user_defined_field5: str
    card_token: str
    ptlf: str
    token_type: str
    token_data: ApplePayTokenData
    payment_method_details: PaymentMethodDetails

    def __init__(self):
        super().__init__(PaymentSourceType.KNET)


class RequestMultiBancoSource(RequestSource):
    payment_country: Country
    account_holder_name: str
    billing_descriptor: str

    def __init__(self):
        super().__init__(PaymentSourceType.MULTIBANCO)


class RequestOxxoSource(RequestSource):
    integration_type: IntegrationType = IntegrationType.REDIRECT
    country: Country
    payer: Payer
    description: str

    def __init__(self):
        super().__init__(PaymentSourceType.OXXO)


class RequestP24Source(RequestSource):
    payment_country: Country
    account_holder_name: str
    account_holder_email: str
    billing_descriptor: str

    def __init__(self):
        super().__init__(PaymentSourceType.P24)


class RequestPagoFacilSource(RequestSource):
    integration_type: IntegrationType = IntegrationType.REDIRECT
    country: Country
    payer: Payer
    description: str

    def __init__(self):
        super().__init__(PaymentSourceType.PAGOFACIL)


class RequestPayPalSource(RequestSource):
    invoice_number: str
    recipient_name: str
    logo_url: str
    stc: dict

    def __init__(self):
        super().__init__(PaymentSourceType.PAYPAL)


class RequestPoliSource(RequestSource):
    def __init__(self):
        super().__init__(PaymentSourceType.POLI)


class RequestQPaySource(RequestSource):
    quantity: int
    description: str
    language: str
    national_id: str

    def __init__(self):
        super().__init__(PaymentSourceType.QPAY)


class RequestRapiPagoSource(RequestSource):
    integration_type: IntegrationType = IntegrationType.REDIRECT
    country: Country
    payer: Payer
    description: str

    def __init__(self):
        super().__init__(PaymentSourceType.RAPIPAGO)


class RequestSepaSource(RequestSource):
    id: str

    def __init__(self):
        super().__init__(PaymentSourceType.ID)


class RequestSofortSource(RequestSource):
    countryCode: Country
    languageCode: str

    def __init__(self):
        super().__init__(PaymentSourceType.SOFORT)



from __future__ import absolute_import

from warnings import warn

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.risk.risk import PreAuthenticationAssessmentRequest, PreCaptureAssessmentRequest


class RiskClient(Client):
    __PRE_AUTHENTICATION_PATH = 'risk/assessments/pre-authentication'
    __PRE_CAPTURE_PATH = 'risk/assessments/pre-capture'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)
        warn(
            'Risk endpoints are no longer supported officially, This module will be removed in a future release.',
            DeprecationWarning,
            stacklevel=2)

    def request_pre_authentication_risk_scan(self,
                                             pre_authentication_assessment_request: PreAuthenticationAssessmentRequest):
        return self._api_client.post(self.__PRE_AUTHENTICATION_PATH, self._sdk_authorization(),
                                     pre_authentication_assessment_request)

    def request_pre_capture_risk_scan(self, pre_capture_assessment_request: PreCaptureAssessmentRequest):
        return self._api_client.post(self.__PRE_CAPTURE_PATH, self._sdk_authorization(), pre_capture_assessment_request)



from enum import Enum

from checkout_sdk.common.enums import Currency


class ForexSource(str, Enum):
    VISA = 'visa'
    MASTERCARD = 'mastercard'


class QuoteRequest:
    source_currency: Currency
    source_amount: int
    destination_currency: Currency
    destination_amount: int
    process_channel_id: str


class RatesQueryFilter:
    product: str
    source: ForexSource
    currency_pairs: str
    process_channel_id: str



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.payments.sessions.sessions import (
    PaymentSessionsRequest, PaymentSessionWithPaymentRequest, SubmitPaymentSessionRequest
)


class PaymentSessionsClient(Client):
    __PAYMENT_SESSIONS_PATH = 'payment-sessions'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def create_payment_sessions(self, payment_sessions_request: PaymentSessionsRequest):
        return self._api_client.post(self.__PAYMENT_SESSIONS_PATH, self._sdk_authorization(),
                                     payment_sessions_request)

    def create_payment_session_with_payment(
            self, payment_session_with_payment_request: PaymentSessionWithPaymentRequest):
        return self._api_client.post(self.__PAYMENT_SESSIONS_PATH + '/complete', self._sdk_authorization(),
                                     payment_session_with_payment_request)

    def submit_payment_session(self, session_id: str,
                               submit_payment_session_request: SubmitPaymentSessionRequest):
        return self._api_client.post(self.__PAYMENT_SESSIONS_PATH + '/' + session_id + '/submit',
                                     self._sdk_authorization(),
                                     submit_payment_session_request)



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.disputes.disputes import DisputesQueryFilter, DisputeEvidenceRequest
from checkout_sdk.files.files_client import FilesClient


class DisputesClient(FilesClient):
    __DISPUTES_PATH = 'disputes'
    __ACCEPT_PATH = 'accept'
    __EVIDENCE_PATH = 'evidence'
    __SUBMITTED_PATH = 'submitted'
    __SCHEME_FILES_PATH = "schemefiles"

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def query(self, query: DisputesQueryFilter):
        return self._api_client.get(self.__DISPUTES_PATH, self._sdk_authorization(),
                                    query)

    def get_dispute_details(self, dispute_id: str):
        return self._api_client.get(self.build_path(self.__DISPUTES_PATH, dispute_id), self._sdk_authorization())

    def accept(self, dispute_id: str):
        return self._api_client.post(self.build_path(self.__DISPUTES_PATH, dispute_id, self.__ACCEPT_PATH),
                                     self._sdk_authorization())

    def put_evidence(self, dispute_id: str, dispute_evidence_request: DisputeEvidenceRequest):
        return self._api_client.put(self.build_path(self.__DISPUTES_PATH, dispute_id, self.__EVIDENCE_PATH),
                                    self._sdk_authorization(), dispute_evidence_request)

    def get_evidence(self, dispute_id: str):
        return self._api_client.get(self.build_path(self.__DISPUTES_PATH, dispute_id, self.__EVIDENCE_PATH),
                                    self._sdk_authorization())

    def submit_evidence(self, dispute_id: str):
        return self._api_client.post(self.build_path(self.__DISPUTES_PATH, dispute_id, self.__EVIDENCE_PATH),
                                     self._sdk_authorization())

    def get_compiled_submitted_evidence(self, dispute_id: str):
        return self._api_client.get(self.build_path(
            self.__DISPUTES_PATH, dispute_id,
            self.__EVIDENCE_PATH,
            self.__SUBMITTED_PATH),
            self._sdk_authorization())

    def get_dispute_scheme_files(self, dispute_id: str):
        return self._api_client.get(self.build_path(self.__DISPUTES_PATH, dispute_id, self.__SCHEME_FILES_PATH),
                                    self._sdk_authorization())



from __future__ import absolute_import

from warnings import warn

from checkout_sdk.accounts.accounts import OnboardEntityRequest, UpdateScheduleRequest, AccountsPaymentInstrument, \
    PaymentInstrumentRequest, PaymentInstrumentsQuery, UpdatePaymentInstrumentRequest
from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.common.enums import Currency
from checkout_sdk.files.files import FileRequest


class AccountsClient(Client):
    __ACCOUNTS_PATH = 'accounts'
    __INSTRUMENTS_PATH = 'instruments'
    __ENTITIES_PATH = 'entities'
    __FILES_PATH = 'files'
    __PAYOUT_SCHEDULES_PATH = 'payout-schedules'
    __PAYMENT_INSTRUMENTS_PATH = 'payment-instruments'

    def __init__(self, api_client: ApiClient,
                 files_client: ApiClient,
                 configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY_OR_OAUTH)
        self.__files_client = files_client

    def upload_file(self, file_request: FileRequest):
        return self.__files_client.submit_file(
            self.__FILES_PATH,
            self._sdk_authorization(),
            file_request,
            multipart_file='path')

    def create_entity(self, onboard_entity_request: OnboardEntityRequest):
        return self._api_client.post(
            self.build_path(self.__ACCOUNTS_PATH, self.__ENTITIES_PATH),
            self._sdk_authorization(),
            onboard_entity_request)

    def retrieve_payment_instrument_details(self, entity_id: str, payment_instrument_id: str):
        return self._api_client.get(
            self.build_path(self.__ACCOUNTS_PATH, self.__ENTITIES_PATH, entity_id, self.__PAYMENT_INSTRUMENTS_PATH,
                            payment_instrument_id),
            self._sdk_authorization()
        )

    def get_entity(self, entity_id: str):
        return self._api_client.get(
            self.build_path(self.__ACCOUNTS_PATH, self.__ENTITIES_PATH, entity_id),
            self._sdk_authorization())

    def update_entity(self, entity_id: str, onboard_entity_request: OnboardEntityRequest):
        return self._api_client.put(
            self.build_path(self.__ACCOUNTS_PATH, self.__ENTITIES_PATH, entity_id),
            self._sdk_authorization(),
            onboard_entity_request)

    def create_payment_instrument(self, entity_id: str, accounts_payment_instrument: AccountsPaymentInstrument):
        warn('Deprecated use add_payment_instrument instead', DeprecationWarning,
             stacklevel=2)
        return self._api_client.post(
            self.build_path(self.__ACCOUNTS_PATH, self.__ENTITIES_PATH, entity_id, self.__INSTRUMENTS_PATH),
            self._sdk_authorization(),
            accounts_payment_instrument)

    def add_payment_instrument(self, entity_id: str, payment_instrument_request: PaymentInstrumentRequest):
        return self._api_client.post(
            self.build_path(self.__ACCOUNTS_PATH, self.__ENTITIES_PATH, entity_id, self.__PAYMENT_INSTRUMENTS_PATH),
            self._sdk_authorization(),
            payment_instrument_request
        )

    def update_payment_instrument(self,
                                  entity_id: str,
                                  instrument_id: str,
                                  update_payment_instrument_request: UpdatePaymentInstrumentRequest):
        return self._api_client.patch(
            self.build_path(self.__ACCOUNTS_PATH,
                            self.__ENTITIES_PATH,
                            entity_id,
                            self.__PAYMENT_INSTRUMENTS_PATH,
                            instrument_id),
            self._sdk_authorization(),
            update_payment_instrument_request
        )

    def query_payment_instruments(self, entity_id: str, query: PaymentInstrumentsQuery = None):
        return self._api_client.get(
            self.build_path(self.__ACCOUNTS_PATH, self.__ENTITIES_PATH, entity_id, self.__PAYMENT_INSTRUMENTS_PATH),
            self._sdk_authorization(),
            query
        )

    def retrieve_payout_schedule(self, entity_id: str):
        return self._api_client.get(
            self.build_path(self.__ACCOUNTS_PATH, self.__ENTITIES_PATH, entity_id, self.__PAYOUT_SCHEDULES_PATH),
            self._sdk_authorization())

    def update_payout_schedule(self, entity_id: str, currency: Currency,
                               update_schedule_request: UpdateScheduleRequest):
        return self._api_client.put(
            self.build_path(self.__ACCOUNTS_PATH, self.__ENTITIES_PATH, entity_id, self.__PAYOUT_SCHEDULES_PATH),
            self._sdk_authorization(),
            {currency: update_schedule_request})



from datetime import datetime


class DisputesQueryFilter:
    limit: int
    skip: int
    from_: datetime
    to: datetime
    id: str
    statuses: str
    payment_id: str
    payment_reference: str
    payment_arn: str
    this_channel_only: bool  # Only available for CS2
    entity_ids: str
    sub_entity_ids: str
    payment_mcc: str


class DisputeEvidenceRequest:
    proof_of_delivery_or_service_file: str
    proof_of_delivery_or_service_text: str
    invoice_or_receipt_file: str
    invoice_or_receipt_text: str
    invoice_showing_distinct_transactions_file: str
    invoice_showing_distinct_transactions_text: str
    customer_communication_file: str
    customer_communication_text: str
    refund_or_cancellation_policy_file: str
    refund_or_cancellation_policy_text: str
    recurring_transaction_agreement_file: str
    recurring_transaction_agreement_text: str
    additional_evidence_file: str
    additional_evidence_text: str
    proof_of_delivery_or_service_date_file: str
    proof_of_delivery_or_service_date_text: str



class ResponseWrapper:

    def __init__(self, http_metadata=None, data=None):
        if http_metadata is not None:
            setattr(self, 'http_metadata', http_metadata)
        if data is not None:
            if isinstance(data, str):
                setattr(self, 'contents', self._wrap(data))
            elif self._is_collection(data):
                setattr(self, 'items', self._wrap(data))
            else:
                for name, value in data.items():
                    setattr(self, name, self._wrap(value))

    def _wrap(self, value):
        if self._is_collection(value):
            return type(value)([self._wrap(v) for v in value])
        else:
            return ResponseWrapper(None, value) if isinstance(value, dict) else value

    @staticmethod
    def _is_collection(value):
        return isinstance(value, (tuple, list, set, frozenset))



from __future__ import absolute_import

from checkout_sdk.accounts.accounts_client import AccountsClient
from checkout_sdk.api_client import ApiClient
from checkout_sdk.balances.balances_client import BalancesClient
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.customers.customers_client import CustomersClient
from checkout_sdk.disputes.disputes_client import DisputesClient
from checkout_sdk.financial.financial_client import FinancialClient
from checkout_sdk.forex.forex_client import ForexClient
from checkout_sdk.checkout_apm_api import CheckoutApmApi
from checkout_sdk.instruments.instruments_client import InstrumentsClient
from checkout_sdk.issuing.issuing_client import IssuingClient
from checkout_sdk.payments.contexts.contexts_client import PaymentContextsClient
from checkout_sdk.payments.sessions.sessions_client import PaymentSessionsClient
from checkout_sdk.payments.hosted.hosted_payments_client import HostedPaymentsClient
from checkout_sdk.payments.links.payments_client import PaymentsLinksClient
from checkout_sdk.payments.payments_client import PaymentsClient
from checkout_sdk.risk.risk_client import RiskClient
from checkout_sdk.sessions.sessions_client import SessionsClient
from checkout_sdk.tokens.tokens_client import TokensClient
from checkout_sdk.transfers.transfers_client import TransfersClient
from checkout_sdk.workflows.workflows_client import WorkflowsClient
from checkout_sdk.reports.reports_client import ReportsClient
from checkout_sdk.metadata.metadata_client import CardMetadataClient
from checkout_sdk.forward.forward_client import ForwardClient


def _base_api_client(configuration: CheckoutConfiguration) -> ApiClient:
    base_uri = configuration.environment.base_uri
    subdomain = configuration.environment_subdomain

    if subdomain is not None and subdomain.base_uri is not None:
        base_uri = subdomain.base_uri

    return ApiClient(configuration, base_uri)


def _files_api_client(configuration: CheckoutConfiguration) -> ApiClient:
    return ApiClient(configuration, configuration.environment.files_uri)


def _transfers_api_client(configuration: CheckoutConfiguration) -> ApiClient:
    return ApiClient(configuration, configuration.environment.transfers_uri)


def _balances_api_client(configuration: CheckoutConfiguration) -> ApiClient:
    return ApiClient(configuration, configuration.environment.balances_uri)


class CheckoutApi(CheckoutApmApi):

    def __init__(self, configuration: CheckoutConfiguration):
        base_api_client = _base_api_client(configuration)
        super().__init__(base_api_client, configuration)
        self.tokens = TokensClient(api_client=base_api_client, configuration=configuration)
        self.customers = CustomersClient(api_client=base_api_client, configuration=configuration)
        self.instruments = InstrumentsClient(api_client=base_api_client, configuration=configuration)
        self.payments = PaymentsClient(api_client=base_api_client, configuration=configuration)
        self.sessions = SessionsClient(api_client=base_api_client, configuration=configuration)
        self.disputes = DisputesClient(api_client=base_api_client, configuration=configuration)
        self.forex = ForexClient(api_client=base_api_client, configuration=configuration)
        self.hosted_payments = HostedPaymentsClient(api_client=base_api_client, configuration=configuration)
        self.payments_links = PaymentsLinksClient(api_client=base_api_client, configuration=configuration)
        self.risk = RiskClient(api_client=base_api_client, configuration=configuration)
        self.workflows = WorkflowsClient(api_client=base_api_client, configuration=configuration)
        self.balances = BalancesClient(api_client=_balances_api_client(configuration), configuration=configuration)
        self.transfers = TransfersClient(api_client=_transfers_api_client(configuration), configuration=configuration)
        self.accounts = AccountsClient(api_client=base_api_client,
                                       files_client=_files_api_client(configuration),
                                       configuration=configuration)
        self.reports = ReportsClient(api_client=base_api_client, configuration=configuration)
        self.card_metadata = CardMetadataClient(api_client=base_api_client, configuration=configuration)
        self.financial = FinancialClient(api_client=base_api_client, configuration=configuration)
        self.issuing = IssuingClient(api_client=base_api_client, configuration=configuration)
        self.contexts = PaymentContextsClient(api_client=base_api_client, configuration=configuration)
        self.payment_sessions = PaymentSessionsClient(api_client=base_api_client, configuration=configuration)
        self.forward = ForwardClient(api_client=base_api_client, configuration=configuration)



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.previous.checkout_apm_api import CheckoutApmApi
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.customers.customers_client_previous import CustomersClient
from checkout_sdk.disputes.disputes_client import DisputesClient
from checkout_sdk.events.events_client import EventsClient
from checkout_sdk.instruments.instruments_client_previous import InstrumentsClient
from checkout_sdk.payments.hosted.hosted_payments_client import HostedPaymentsClient
from checkout_sdk.payments.links.payments_client import PaymentsLinksClient
from checkout_sdk.payments.payments_client_previous import PaymentsClient
from checkout_sdk.reconciliation.reconciliation_client import ReconciliationClient
from checkout_sdk.risk.risk_client import RiskClient
from checkout_sdk.sources.sources_client import SourcesClient
from checkout_sdk.tokens.tokens_client import TokensClient
from checkout_sdk.webhooks.webhooks_client import WebhooksClient


class CheckoutApi(CheckoutApmApi):

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client, configuration)
        self.tokens = TokensClient(api_client=api_client, configuration=configuration)
        self.sources = SourcesClient(api_client=api_client, configuration=configuration)
        self.customers = CustomersClient(api_client=api_client, configuration=configuration)
        self.instruments = InstrumentsClient(api_client=api_client, configuration=configuration)
        self.payments = PaymentsClient(api_client=api_client, configuration=configuration)
        self.disputes = DisputesClient(api_client=api_client, configuration=configuration)
        self.hosted_payments = HostedPaymentsClient(api_client=api_client, configuration=configuration)
        self.payments_links = PaymentsLinksClient(api_client=api_client, configuration=configuration)
        self.risk = RiskClient(api_client=api_client, configuration=configuration)
        self.webhooks = WebhooksClient(api_client=api_client, configuration=configuration)
        self.events = EventsClient(api_client=api_client, configuration=configuration)
        self.reconciliation = ReconciliationClient(api_client=api_client, configuration=configuration)



from __future__ import absolute_import

from checkout_sdk.exception import CheckoutAuthorizationException
from checkout_sdk.platform_type import PlatformType


class SdkAuthorization:

    def __init__(self, platform_type, credential):
        self.platform_type = platform_type
        self.credential = credential

    def get_authorization_header(self):
        if self.platform_type in (PlatformType.PREVIOUS, PlatformType.CUSTOM):
            return self.credential
        if self.platform_type in (PlatformType.DEFAULT, PlatformType.OAUTH):
            return 'Bearer ' + self.credential
        raise CheckoutAuthorizationException('Invalid platform type')



from datetime import datetime

from checkout_sdk.common.common import Address, Phone, CustomerRequest
from checkout_sdk.common.enums import PaymentSourceType, Currency


class RiskPayment:
    psp: str
    id: str


class RiskShippingDetails:
    address: Address


class Location:
    latitude: str
    longitude: str


class Device:
    ip: str
    location: Location
    os: str
    type: str
    model: str
    date: datetime
    user_agent: str
    fingerprint: str


class AuthenticationResult:
    attempted: bool
    challenged: bool
    succeeded: bool
    liability_shifted: bool
    method: str
    version: str


class AuthorizationResult:
    avs_code: str
    cvv_result: str


class RiskPaymentRequestSource:
    type: PaymentSourceType

    def __init__(self, type_p: PaymentSourceType):
        self.type = type_p


class RiskRequestTokenSource(RiskPaymentRequestSource):
    token: str
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentSourceType.TOKEN)


class IdSourcePrism(RiskPaymentRequestSource):
    id: str
    cvv: str

    def __init__(self):
        super().__init__(PaymentSourceType.ID)


class CardSourcePrism(RiskPaymentRequestSource):
    number: str
    expiry_month: int
    expiry_year: int
    name: str
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentSourceType.CARD)


class CustomerSourcePrism(RiskPaymentRequestSource):
    id: str

    def __init__(self):
        super().__init__(PaymentSourceType.CUSTOMER)


class PreAuthenticationAssessmentRequest:
    date: datetime
    source: RiskPaymentRequestSource
    customer: CustomerRequest
    payment: RiskPayment
    shipping: RiskShippingDetails
    reference: str
    description: str
    amount: int
    currency: Currency
    device: Device
    metadata: dict


class PreCaptureAssessmentRequest:
    assessment_id: str
    date: datetime
    source: RiskPaymentRequestSource
    customer: CustomerRequest
    amount: int
    currency: Currency
    payment: RiskPayment
    shipping: RiskShippingDetails
    device: Device
    metadata: dict
    authentication_result: AuthenticationResult
    authorization_result: AuthorizationResult



from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.common.common import QueryFilterDateRange
from checkout_sdk.reconciliation.reconciliation import ReconciliationQueryFilter


class ReconciliationClient(Client):
    __REPORTING_PATH = 'reporting'
    __PAYMENTS_PATH = 'payments'
    __DOWNLOAD_PATH = 'download'
    __STATEMENTS_PATH = 'statements'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def query_payments_report(self, query: ReconciliationQueryFilter):
        return self._api_client.get(self.build_path(self.__REPORTING_PATH, self.__PAYMENTS_PATH),
                                    self._sdk_authorization(),
                                    query)

    def single_payment_report(self, payment_id: str):
        return self._api_client.get(self.build_path(self.__REPORTING_PATH, self.__PAYMENTS_PATH, payment_id),
                                    self._sdk_authorization())

    def query_statements_report(self, query: QueryFilterDateRange):
        return self._api_client.get(self.build_path(self.__REPORTING_PATH, self.__STATEMENTS_PATH),
                                    self._sdk_authorization(),
                                    query)

    def retrieve_csv_payment_report(self, query: QueryFilterDateRange):
        return self._api_client.get(self.build_path(self.__REPORTING_PATH, self.__PAYMENTS_PATH, self.__DOWNLOAD_PATH),
                                    self._sdk_authorization(),
                                    query)

    def retrieve_csv_single_statement_report(self, statement_id: str):
        return self._api_client.get(
            self.build_path(self.__REPORTING_PATH, self.__STATEMENTS_PATH, statement_id, self.__PAYMENTS_PATH,
                            self.__DOWNLOAD_PATH),
            self._sdk_authorization())

    def retrieve_csv_statements_report(self, query: QueryFilterDateRange):
        return self._api_client.get(
            self.build_path(self.__REPORTING_PATH, self.__STATEMENTS_PATH, self.__DOWNLOAD_PATH),
            self._sdk_authorization(),
            query)



from __future__ import absolute_import

from enum import Enum


class Currency(str, Enum):
    ALL = 'ALL'
    STN = 'STN'
    EEK = 'EEK'
    BHD = 'BHD'
    SCR = 'SCR'
    DJF = 'DJF'
    EGP = 'EGP'
    MDL = 'MDL'
    MZN = 'MZN'
    BND = 'BND'
    ZMK = 'ZMK'
    SHP = 'SHP'
    LBP = 'LBP'
    AWG = 'AWG'
    JMD = 'JMD'
    KES = 'KES'
    BYN = 'BYN'
    KHR = 'KHR'
    LAK = 'LAK'
    MVR = 'MVR'
    AOA = 'AOA'
    TJS = 'TJS'
    SVC = 'SVC'
    GNF = 'GNF'
    BRL = 'BRL'
    MOP = 'MOP'
    BOB = 'BOB'
    CDF = 'CDF'
    NAD = 'NAD'
    LYD = 'LYD'
    VUV = 'VUV'
    QAR = 'QAR'
    CLP = 'CLP'
    HRK = 'HRK'
    ISK = 'ISK'
    FKP = 'FKP'
    XCD = 'XCD'
    NOK = 'NOK'
    CUP = 'CUP'
    VND = 'VND'
    PEN = 'PEN'
    KMF = 'KMF'
    LVL = 'LVL'
    MMK = 'MMK'
    TRY = 'TRY'
    VEF = 'VEF'
    AUD = 'AUD'
    TWD = 'TWD'
    PKR = 'PKR'
    SLL = 'SLL'
    BGN = 'BGN'
    LRD = 'LRD'
    LKR = 'LKR'
    XAF = 'XAF'
    JOD = 'JOD'
    ANG = 'ANG'
    BSD = 'BSD'
    CAD = 'CAD'
    GIP = 'GIP'
    MNT = 'MNT'
    LTL = 'LTL'
    BBD = 'BBD'
    CLF = 'CLF'
    BWP = 'BWP'
    COP = 'COP'
    PHP = 'PHP'
    HUF = 'HUF'
    FJD = 'FJD'
    MWK = 'MWK'
    THB = 'THB'
    XPF = 'XPF'
    RSD = 'RSD'
    SAR = 'SAR'
    UYU = 'UYU'
    BZD = 'BZD'
    SYP = 'SYP'
    GMD = 'GMD'
    SZL = 'SZL'
    SBD = 'SBD'
    ETB = 'ETB'
    CHF = 'CHF'
    MXN = 'MXN'
    ARS = 'ARS'
    GTQ = 'GTQ'
    GHS = 'GHS'
    NIO = 'NIO'
    JPY = 'JPY'
    BDT = 'BDT'
    UZS = 'UZS'
    SOS = 'SOS'
    BTN = 'BTN'
    NZD = 'NZD'
    TZS = 'TZS'
    IQD = 'IQD'
    MGA = 'MGA'
    DZD = 'DZD'
    GYD = 'GYD'
    USD = 'USD'
    KWD = 'KWD'
    CNY = 'CNY'
    PYG = 'PYG'
    SGD = 'SGD'
    KZT = 'KZT'
    PGK = 'PGK'
    AMD = 'AMD'
    GBP = 'GBP'
    AFN = 'AFN'
    CRC = 'CRC'
    XOF = 'XOF'
    YER = 'YER'
    MRU = 'MRU'
    DKK = 'DKK'
    TOP = 'TOP'
    INR = 'INR'
    SDG = 'SDG'
    DOP = 'DOP'
    ZWL = 'ZWL'
    UGX = 'UGX'
    SEK = 'SEK'
    LSL = 'LSL'
    MYR = 'MYR'
    TMT = 'TMT'
    OMR = 'OMR'
    BMD = 'BMD'
    KRW = 'KRW'
    HKD = 'HKD'
    KGS = 'KGS'
    BAM = 'BAM'
    NGN = 'NGN'
    ILS = 'ILS'
    MUR = 'MUR'
    RON = 'RON'
    TND = 'TND'
    AED = 'AED'
    PAB = 'PAB'
    NPR = 'NPR'
    TTD = 'TTD'
    RWF = 'RWF'
    HTG = 'HTG'
    IDR = 'IDR'
    EUR = 'EUR'
    KYD = 'KYD'
    IRR = 'IRR'
    KPW = 'KPW'
    MKD = 'MKD'
    SRD = 'SRD'
    HNL = 'HNL'
    AZN = 'AZN'
    ERN = 'ERN'
    CZK = 'CZK'
    CVE = 'CVE'
    BIF = 'BIF'
    MAD = 'MAD'
    RUB = 'RUB'
    UAH = 'UAH'
    WST = 'WST'
    PLN = 'PLN'
    ZAR = 'ZAR'
    GEL = 'GEL'
    ZMW = 'ZMW'


class Country(str, Enum):
    AF = 'AF'
    AX = 'AX'
    AL = 'AL'
    DZ = 'DZ'
    AS = 'AS'
    AD = 'AD'
    AO = 'AO'
    AI = 'AI'
    AG = 'AG'
    AR = 'AR'
    AM = 'AM'
    AW = 'AW'
    AC = 'AC'
    AU = 'AU'
    AQ = 'AQ'
    AT = 'AT'
    AZ = 'AZ'
    BS = 'BS'
    BH = 'BH'
    BD = 'BD'
    BB = 'BB'
    BY = 'BY'
    BE = 'BE'
    BZ = 'BZ'
    BJ = 'BJ'
    BM = 'BM'
    BT = 'BT'
    BO = 'BO'
    BQ = 'BQ'
    BA = 'BA'
    BW = 'BW'
    BV = 'BV'
    BR = 'BR'
    IO = 'IO'
    VG = 'VG'
    BN = 'BN'
    BG = 'BG'
    BF = 'BF'
    BI = 'BI'
    KH = 'KH'
    CM = 'CM'
    CA = 'CA'
    CV = 'CV'
    KY = 'KY'
    CF = 'CF'
    TD = 'TD'
    CL = 'CL'
    CN = 'CN'
    TW = 'TW'
    CX = 'CX'
    PF = 'PF'
    CC = 'CC'
    CO = 'CO'
    KM = 'KM'
    CG = 'CG'
    CD = 'CD'
    CK = 'CK'
    CR = 'CR'
    CI = 'CI'
    HR = 'HR'
    CU = 'CU'
    CW = 'CW'
    CY = 'CY'
    CZ = 'CZ'
    DK = 'DK'
    DJ = 'DJ'
    DM = 'DM'
    DO = 'DO'
    EC = 'EC'
    EG = 'EG'
    SV = 'SV'
    GQ = 'GQ'
    ER = 'ER'
    EE = 'EE'
    SZ = 'SZ'
    ET = 'ET'
    FK = 'FK'
    FO = 'FO'
    FJ = 'FJ'
    FI = 'FI'
    FR = 'FR'
    GF = 'GF'
    TF = 'TF'
    GA = 'GA'
    GM = 'GM'
    GE = 'GE'
    DE = 'DE'
    GH = 'GH'
    GI = 'GI'
    GR = 'GR'
    GL = 'GL'
    GD = 'GD'
    GP = 'GP'
    GU = 'GU'
    GT = 'GT'
    GG = 'GG'
    GN = 'GN'
    GW = 'GW'
    GY = 'GY'
    HT = 'HT'
    HM = 'HM'
    HN = 'HN'
    HK = 'HK'
    HU = 'HU'
    IS = 'IS'
    IN = 'IN'
    ID = 'ID'
    IR = 'IR'
    IQ = 'IQ'
    IE = 'IE'
    IM = 'IM'
    IL = 'IL'
    IT = 'IT'
    JM = 'JM'
    JP = 'JP'
    JE = 'JE'
    JO = 'JO'
    KZ = 'KZ'
    KE = 'KE'
    KI = 'KI'
    KP = 'KP'
    KR = 'KR'
    KW = 'KW'
    KG = 'KG'
    LA = 'LA'
    LV = 'LV'
    LB = 'LB'
    LS = 'LS'
    LR = 'LR'
    LY = 'LY'
    LI = 'LI'
    LT = 'LT'
    LU = 'LU'
    MO = 'MO'
    MK = 'MK'
    MG = 'MG'
    MW = 'MW'
    MY = 'MY'
    MV = 'MV'
    ML = 'ML'
    MT = 'MT'
    MH = 'MH'
    MQ = 'MQ'
    MR = 'MR'
    MU = 'MU'
    YT = 'YT'
    MX = 'MX'
    FM = 'FM'
    MD = 'MD'
    MC = 'MC'
    MN = 'MN'
    ME = 'ME'
    MS = 'MS'
    MA = 'MA'
    MZ = 'MZ'
    MM = 'MM'
    NA = 'NA'
    NR = 'NR'
    NP = 'NP'
    NL = 'NL'
    AN = 'AN'
    NC = 'NC'
    NZ = 'NZ'
    NI = 'NI'
    NE = 'NE'
    NG = 'NG'
    NU = 'NU'
    NF = 'NF'
    MP = 'MP'
    NO = 'NO'
    OM = 'OM'
    PK = 'PK'
    PW = 'PW'
    PA = 'PA'
    PG = 'PG'
    PY = 'PY'
    PE = 'PE'
    PH = 'PH'
    PN = 'PN'
    PL = 'PL'
    PT = 'PT'
    PR = 'PR'
    QA = 'QA'
    RE = 'RE'
    RO = 'RO'
    RU = 'RU'
    RW = 'RW'
    BL = 'BL'
    SH = 'SH'
    KN = 'KN'
    LC = 'LC'
    MF = 'MF'
    PM = 'PM'
    VC = 'VC'
    WS = 'WS'
    SM = 'SM'
    ST = 'ST'
    SA = 'SA'
    SN = 'SN'
    RS = 'RS'
    SC = 'SC'
    SL = 'SL'
    SG = 'SG'
    SX = 'SX'
    SK = 'SK'
    SI = 'SI'
    SB = 'SB'
    SO = 'SO'
    ZA = 'ZA'
    GS = 'GS'
    SS = 'SS'
    ES = 'ES'
    LK = 'LK'
    SD = 'SD'
    SR = 'SR'
    SJ = 'SJ'
    SE = 'SE'
    CH = 'CH'
    SY = 'SY'
    TJ = 'TJ'
    TZ = 'TZ'
    TH = 'TH'
    TL = 'TL'
    TG = 'TG'
    TK = 'TK'
    TO = 'TO'
    TT = 'TT'
    TA = 'TA'
    TN = 'TN'
    TR = 'TR'
    TM = 'TM'
    TC = 'TC'
    TV = 'TV'
    VI = 'VI'
    UG = 'UG'
    UA = 'UA'
    AE = 'AE'
    GB = 'GB'
    US = 'US'
    UY = 'UY'
    UZ = 'UZ'
    VU = 'VU'
    VA = 'VA'
    VE = 'VE'
    VN = 'VN'
    UM = 'UM'
    WF = 'WF'
    EH = 'EH'
    YE = 'YE'
    ZM = 'ZM'
    ZW = 'ZW'
    PS = 'PS'
    QZ = 'QZ'


class PaymentSourceType(str, Enum):
    CARD = 'card'
    ID = 'id'
    NETWORK_TOKEN = 'network_token'
    TOKEN = 'token'
    CUSTOMER = 'customer'
    D_LOCAL = 'dlocal'
    KLARNA = 'klarna'
    CURRENCY_ACCOUNT = 'currency_account'
    BOLETO = 'boleto'
    FAWRY = 'fawry'
    GIROPAY = 'giropay'
    IDEAL = 'ideal'
    OXXO = 'oxxo'
    PAGOFACIL = 'pagofacil'
    RAPIPAGO = 'rapipago'
    SOFORT = 'sofort'
    KNET = 'knet'
    QPAY = 'qpay'
    ALIPAY = 'alipay'
    PAYPAL = 'paypal'
    MULTIBANCO = 'multibanco'
    EPS = 'eps'
    ILLICADO = 'illicado'
    POLI = 'poli'
    P24 = 'p24'
    BENEFITPAY = 'benefitpay'
    BANCONTACT = 'bancontact'
    TAMARA = 'tamara'
    PROVIDER_TOKEN = 'provider_token'
    BANK_ACCOUNT = 'bank_account'
    WECHATPAY = 'wechatpay'
    ALIPAY_HK = 'alipay_hk'
    ALIPAY_CN = 'alipay_cn'
    ALIPAY_PLUS = 'alipay_plus'
    GCASH = 'gcash'
    DANA = 'dana'
    KAKAOPAY = 'kakaopay'
    TRUEMONEY = 'truemoney'
    TNG = 'tng'
    AFTERPAY = 'afterpay'
    BENEFIT = 'benefit'
    MBWAY = 'mbway'
    POSTFINANCE = 'postfinance'
    STCPAY = 'stcpay'
    ALMA = 'alma'
    TRUSTLY = 'trustly'
    CVCONNECT = 'cvconnect'
    SEPA = 'sepa'
    ACH = 'ach'
    BIZUM = 'bizum'
    OCTOPUS = 'octopus'
    PLAID = 'plaid'
    SEQURA = 'sequra'


class ChallengeIndicator(str, Enum):
    NO_PREFERENCE = 'no_preference'
    NO_CHALLENGE_REQUESTED = 'no_challenge_requested'
    CHALLENGE_REQUESTED = 'challenge_requested'
    CHALLENGE_REQUESTED_MANDATE = 'challenge_requested_mandate'


class InstrumentType(str, Enum):
    BANK_ACCOUNT = 'bank_account'
    TOKEN = 'token'
    CARD = 'card'
    SEPA = 'sepa'
    CARD_TOKEN = 'card_token'


class AccountType(str, Enum):
    SAVINGS = 'savings'
    CURRENT = 'current'
    CASH = 'cash'


class AccountHolderType(str, Enum):
    INDIVIDUAL = 'individual'
    CORPORATE = 'corporate'
    INSTRUMENT = 'instrument'


class AccountHolderIdentificationType(str, Enum):
    PASSPORT = 'passport'
    DRIVING_LICENSE = 'driving_licence'
    NATIONAL_ID = 'national_id'
    COMPANY_REGISTRATION = 'company_registration'
    TAX_ID = 'tax_id'


class DocumentType(str, Enum):
    PASSPORT = 'passport'
    NATIONAL_IDENTITY_CARD = 'national_identity_card'
    DRIVING_LICENSE = 'driving_license'
    CITIZEN_CARD = 'citizen_card'
    RESIDENCE_PERMIT = 'residence_permit'
    ELECTORAL_ID = 'electoral_id'


class AccountChangeIndicatorType(str, Enum):
    THIS_TRANSACTION = "this_transaction"
    LESS_THAN_THIRTY_DAYS = "less_than_thirty_days"
    THIRTY_TO_SIXTY_DAYS = "thirty_to_sixty_days"
    MORE_THAN_SIXTY_DAYS = "more_than_sixty_days"


class AccountPasswordChangeIndicatorType(str, Enum):
    NO_CHANGE = "no_change"
    THIS_TRANSACTION = "this_transaction"
    LESS_THAN_THIRTY_DAYS = "less_than_thirty_days"
    THIRTY_TO_SIXTY_DAYS = "thirty_to_sixty_days"
    MORE_THAN_SIXTY_DAYS = "more_than_sixty_days"


class AccountTypeCardProductType(str, Enum):
    CREDIT = "credit"
    DEBIT = "debit"
    NOT_APPLICABLE = "not_applicable"


class CardholderAccountAgeIndicatorType(str, Enum):
    LESS_THAN_THIRTY_DAYS = "less_than_thirty_days"
    MORE_THAN_SIXTY_DAYS = "more_than_sixty_days"
    NO_ACCOUNT = "no_account"
    THIRTY_TO_SIXTY_DAYS = "thirty_to_sixty_days"
    THIS_TRANSACTION = "this_transaction"



from enum import Enum


class CardMetadataSourceType(str, Enum):
    BIN = 'bin'
    CARD = 'card'
    ID = 'id'
    TOKEN = 'token'


class CardMetadataFormatType(str, Enum):
    BASIC = 'basic'
    CARD_PAYOUTS = 'card_payouts'


class CardMetadataRequestSource:
    type: CardMetadataSourceType

    def __init__(self, type_p: CardMetadataSourceType):
        self.type = type_p


class CardMetadataBinSource(CardMetadataRequestSource):
    bin: str

    def __init__(self):
        super().__init__(CardMetadataSourceType.BIN)


class CardMetadataCardSource(CardMetadataRequestSource):
    number: str

    def __init__(self):
        super().__init__(CardMetadataSourceType.CARD)


class CardMetadataIdSource(CardMetadataRequestSource):
    id: str

    def __init__(self):
        super().__init__(CardMetadataSourceType.ID)


class CardMetadataTokenSource(CardMetadataRequestSource):
    token: str

    def __init__(self):
        super().__init__(CardMetadataSourceType.TOKEN)


class CardMetadataRequest:
    source: CardMetadataRequestSource
    format: CardMetadataFormatType



from datetime import datetime

from deprecated import deprecated

from checkout_sdk.common.common import Address, CustomerRequest, AccountHolder
from checkout_sdk.common.enums import Currency, PaymentSourceType
from checkout_sdk.payments.payments import PaymentRequestSource, PaymentType, ShippingDetails, BillingPlan, \
    ShippingPreference, UserAction


class PaymentContextsPartnerCustomerRiskData:
    key: str
    value: str


class PaymentContextsTicket:
    number: str
    issue_date: datetime
    issuing_carrier_code: str
    travel_package_indicator: str
    travel_agency_name: str
    travel_agency_code: str


class PaymentContextsPassenger:
    first_name: str
    last_name: str
    date_of_birth: datetime
    address: Address


class PaymentContextsFlightLegDetails:
    flight_number: str
    carrier_code: str
    class_of_travelling: str
    departure_airport: str
    departure_date: datetime
    departure_time: str
    arrival_airport: str
    stop_over_code: str
    fare_basis_code: str


class PaymentContextsAirlineData:
    ticket: list  # payment.contexts.PaymentContextsTicket
    passenger: list  # payment.contexts.PaymentContextsPassenger
    flight_leg_details: list  # payment.contexts.PaymentContextsFlightLegDetails


class PaymentContextsProcessing:
    plan: BillingPlan
    shipping_amount: int
    invoice_id: str
    brand_name: str
    locale: str
    shipping_preference: ShippingPreference
    user_action: UserAction
    partner_customer_risk_data: list  # payment.contexts.PaymentContextsPartnerCustomerRiskData
    airline_data: list  # payment.contexts.PaymentContextsAirlineData


class PaymentContextsItems:
    name: str
    quantity: int
    unit_price: int
    reference: str
    total_amount: int
    tax_amount: int
    discount_amount: int
    url: str
    image_url: str


class PaymentContextsRequest:
    source: PaymentRequestSource
    amount: int
    currency: Currency
    payment_type: PaymentType
    authorization_type: str
    capture: bool
    customer: CustomerRequest
    shipping: ShippingDetails
    processing: PaymentContextsProcessing
    processing_channel_id: str
    reference: str
    description: str
    success_url: str
    failure_url: str
    items: list  # payments.contexts.PaymentContextsItems


@deprecated("This class will be removed in the future. Use PaymentContextPaypalSource instead")
class PaymentContextPayPalSource(PaymentRequestSource):

    def __init__(self):
        super().__init__(PaymentSourceType.PAYPAL)


class PaymentContextPaypalSource(PaymentRequestSource):

    def __init__(self):
        super().__init__(PaymentSourceType.PAYPAL)


class PaymentContextKlarnaSource(PaymentRequestSource):
    account_holder: AccountHolder

    def __init__(self):
        super().__init__(PaymentSourceType.KLARNA)



from __future__ import absolute_import

from enum import Enum

from checkout_sdk.common.common import Address, Phone, CustomerRequest


class SourceType(str, Enum):
    SEPA = 'sepa'


class MandateType(str, Enum):
    SINGLE = 'single'
    RECURRING = 'recurring'


class SourceRequest:
    reference: str
    phone: Phone
    customer: CustomerRequest

    def __init__(self, _type: SourceType):
        self.type = _type


class SourceData:
    first_name: str
    last_name: str
    account_iban: str
    bic: str
    billing_descriptor: str
    mandate_type: MandateType


class SepaSourceRequest(SourceRequest):
    billing_address: Address
    source_data: SourceData

    def __init__(self):
        super().__init__(SourceType.SEPA)



from checkout_sdk.http_metadata import HttpMetadata


def map_to_http_metadata(response):
    if response is None:
        return None
    http_metadata = HttpMetadata()
    http_metadata.reason_phrase = response.reason
    http_metadata.status_code = response.status_code
    http_metadata.headers = response.headers
    return http_metadata



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.forex.forex import QuoteRequest, RatesQueryFilter


class ForexClient(Client):
    __FOREX_PATH = 'forex'
    __QUOTES_PATH = 'quotes'
    __RATES_PATH = 'rates'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.OAUTH)

    def request_quote(self, quote_request: QuoteRequest):
        return self._api_client.post(self.build_path(self.__FOREX_PATH, self.__QUOTES_PATH),
                                     self._sdk_authorization(), quote_request)

    def get_rates(self, rates_query: RatesQueryFilter):
        return self._api_client.get(self.build_path(self.__FOREX_PATH, self.__RATES_PATH),
                                    self._sdk_authorization(), rates_query)



from datetime import datetime
from enum import Enum

from checkout_sdk.common.enums import AccountHolderIdentificationType, AccountHolderType, Country, AccountType


class Address:
    address_line1: str
    address_line2: str
    city: str
    state: str
    zip: str
    country: Country


class Phone:
    country_code: str
    number: str


class CustomerRequest:
    id: str
    email: str
    name: str
    phone: Phone


class CustomerRetry:
    max_attempts: int


class ResidentialStatusType(str, Enum):
    RESIDENT = 'resident'
    NON_RESIDENT = 'non_resident'


class AccountHolderIdentification:
    type: AccountHolderIdentificationType
    number: str
    issuing_country: Country
    date_of_expiry: str


class AccountHolder:
    type: AccountHolderType
    full_name: str
    first_name: str
    middle_name: str
    last_name: str
    email: str
    gender: str
    company_name: str
    tax_id: str
    date_of_birth: str
    country_of_birth: Country
    residential_status: ResidentialStatusType
    billing_address: Address
    phone: Phone
    identification: AccountHolderIdentification
    account_name_inquiry: bool


class BankDetails:
    name: str
    branch: str
    address: Address


class UpdateCustomerRequest:
    id: str
    default: bool


class MarketplaceCommission:
    amount: int
    percentage: float


class MarketplaceDataSubEntity:
    id: str
    amount: int
    reference: str
    commission: MarketplaceCommission


class MarketplaceData:
    sub_entity_id: str
    sub_entities: list  # MarketplaceDataSubEntity


class Product:
    name: str
    quantity: int
    price: int


class Commission:
    amount: int
    percentage: float


class AmountAllocations:
    id: str
    amount: int
    reference: str
    commission: Commission


class ShippingInfo:
    shipping_company: str
    shipping_method: str
    tracking_number: str
    tracking_uri: str
    return_shipping_company: str
    return_tracking_number: str
    return_tracking_uri: str


class QueryFilterDateRange:
    from_: datetime
    to: datetime


class Destination:
    account_type: AccountType
    account_number: str
    bank_code: str
    branch_code: str
    iban: str
    bban: str
    swift_bic: str
    country: Country
    account_holder: AccountHolder
    bank: BankDetails



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client


class IdealClient(Client):
    __IDEAL_EXTERNAL_PATH = 'ideal-external'
    __ISSUERS_PATH_PATH = 'issuers'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def get_info(self):
        return self._api_client.get(self.build_path(self.__IDEAL_EXTERNAL_PATH), self._sdk_authorization())

    def get_issuers(self):
        return self._api_client.get(self.build_path(self.__IDEAL_EXTERNAL_PATH, self.__ISSUERS_PATH_PATH),
                                    self._sdk_authorization())



from enum import Enum

from checkout_sdk.common.common import Address, Phone


class CardType(str, Enum):
    PHYSICAL = 'physical'
    VIRTUAL = 'virtual'


class LifetimeUnit(str, Enum):
    MONTHS = 'Months'
    YEARS = 'Years'


class RevokeReason(str, Enum):
    EXPIRED = 'expired'
    REPORTED_LOST = 'reported_lost'
    REPORTED_STOLEN = 'reported_stolen'


class SuspendReason(str, Enum):
    SUSPECTED_LOST = 'suspected_lost'
    SUSPECTED_STOLEN = 'suspected_stolen'


class CardLifetime:
    unit: LifetimeUnit
    value: int


class ShippingInstructions:
    recipient_address: str
    shipping_address: Address
    additional_comment: str


class CardRequest:
    type: CardType
    cardholder_id: str
    lifetime: CardLifetime
    reference: str
    card_product_id: str
    display_name: str
    activate_card: bool

    def __init__(self, type_p: CardType):
        self.type = type_p


class PhysicalCardRequest(CardRequest):
    shipping_instructions: ShippingInstructions

    def __init__(self):
        super().__init__(CardType.PHYSICAL)


class VirtualCardRequest(CardRequest):
    is_single_use: bool

    def __init__(self):
        super().__init__(CardType.VIRTUAL)


class SecurityPair:
    question: str
    answer: str


class ThreeDsEnrollmentRequest:
    locale: str
    phone_number: Phone


class SecurityQuestionEnrollmentRequest(ThreeDsEnrollmentRequest):
    security_pair: SecurityPair


class PasswordEnrollmentRequest(ThreeDsEnrollmentRequest):
    password: str


class UpdateThreeDsEnrollmentRequest:
    security_pair: SecurityPair
    password: str
    locale: str
    phone_number: Phone


class CardCredentialsQuery:
    credentials: str


class RevokeRequest:
    reason: RevokeReason


class SuspendRequest:
    reason: SuspendReason



from checkout_sdk.common.common import Phone


class CustomerRequest:
    email: str
    name: str
    phone: Phone
    metadata: dict
    default: str
    instruments: list  # Not available on previous



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.apm.ideal_client import IdealClient
from checkout_sdk.apm.klarna_client import KlarnaClient
from checkout_sdk.apm.sepa_client import SepaClient
from checkout_sdk.checkout_configuration import CheckoutConfiguration


class CheckoutApmApi:

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        self.ideal = IdealClient(api_client=api_client, configuration=configuration)
        self.klarna = KlarnaClient(api_client=api_client, configuration=configuration)
        self.sepa = SepaClient(api_client=api_client, configuration=configuration)



from enum import Enum

from checkout_sdk.common.enums import Currency


class TransactionType(str, Enum):
    PURCHASE = 'purchase'


class AuthorizationType(str, Enum):
    AUTHORIZATION = 'authorization'
    PRE_AUTHORIZATION = 'pre_authorization'


class CardSimulation:
    id: str
    expiry_month: int
    expiry_year: int


class Merchant:
    category_code: str


class TransactionSimulation:
    type: TransactionType
    amount: int
    currency: Currency
    merchant: Merchant
    authorization_type: AuthorizationType


class CardAuthorizationRequest:
    card: CardSimulation
    transaction: TransactionSimulation


class SimulationRequest:
    amount: int



from __future__ import absolute_import

from datetime import datetime
from enum import Enum
from checkout_sdk.common.common import Address, Phone, CustomerRequest
from checkout_sdk.common.enums import Currency, PaymentSourceType, Country
from checkout_sdk.payments.payments import PaymentType, PaymentDestinationType, ShippingDetails, ThreeDsRequest, \
    RiskRequest, PaymentRecipient, ProcessingSettings, BillingDescriptor


class Purpose(str, Enum):
    DONATIONS = 'donations'
    EDUCATION = 'education'
    EMERGENCY_NEED = 'emergency_need'
    EXPATRIATION = 'expatriation'
    FAMILY_SUPPORT = 'family_support'
    GIFTS = 'gifts'
    INCOME = 'income'
    INSURANCE = 'insurance'
    INVESTMENT = 'investment'
    IT_SERVICES = 'it_services'
    LEISURE = 'leisure'
    LOAN_PAYMENT = 'loan_payment'
    OTHER = 'other'
    PENSION = 'pension'
    ROYALTIES = 'royalties'
    SAVINGS = 'savings'
    TRAVEL_AND_TOURISM = 'travel_and_tourism'
    FINANCIAL_SERVICES = 'financial_services'
    MEDICAL_TREATMENT = 'medical_treatment'


class FundTransferType(str, Enum):
    AA = 'AA'
    PP = 'PP'
    FT = 'FT'
    FD = 'FD'
    PD = 'PD'
    LO = 'LO'
    OG = 'OG'


class BillingInformation:
    address: Address
    phone: Phone


class AirlineTicket:
    number: str
    issue_date: str
    issuing_carrier_code: str
    travel_agency_name: str
    travel_agency_code: str


class AirlinePassengerName:
    full_name: str


class AirlinePassenger:
    name: AirlinePassengerName
    date_of_birth: str
    country_code: Country


class AirlineFlightLegDetails:
    flight_number: int
    carrier_code: str
    service_class: str
    departure_date: str
    departure_time: str
    departure_airport: str
    arrival_airport: str
    stopover_code: str
    fare_basis_code: str


class AirlineData:
    ticket: AirlineTicket
    passenger: AirlinePassenger
    flight_leg_details: list  # AirlineFlightLegDetails


# Request Source
class RequestSource:
    type: PaymentSourceType

    def __init__(self, type_p: PaymentSourceType):
        self.type = type_p


class RequestCardSource(RequestSource):
    number: str
    expiry_month: int
    expiry_year: int
    name: str
    cvv: str
    stored: bool
    store_for_future_use: bool
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentSourceType.CARD)


class RequestCustomerSource(RequestSource):
    id: str

    def __init__(self):
        super().__init__(PaymentSourceType.ID)


class RequestDLocalSource(RequestSource):
    number: str
    expiry_month: int
    expiry_year: int
    name: str
    cvv: str
    stored: bool
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentSourceType.D_LOCAL)


class RequestIdSource(RequestSource):
    id: str
    cvv: str
    payment_method: str
    stored: bool
    store_for_future_use: bool

    def __init__(self):
        super().__init__(PaymentSourceType.ID)


class RequestNetworkTokenSource(RequestSource):
    token: str
    expiry_month: int
    expiry_year: int
    token_type: str
    cryptogram: str
    eci: str
    stored: bool
    name: str
    cvv: str
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentSourceType.NETWORK_TOKEN)


class RequestTokenSource(RequestSource):
    token: str
    billing_address: Address
    phone: Phone
    stored: bool
    store_for_future_use: bool

    def __init__(self):
        super().__init__(PaymentSourceType.TOKEN)


# Request Destination
class RequestDestination:
    type: PaymentDestinationType
    first_name: str
    last_name: str

    def __init__(self, type_p: PaymentDestinationType):
        self.type = type_p


class PaymentRequestCardDestination(RequestDestination):
    number: str
    expiry_month: int
    expiry_year: int
    name: str
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentDestinationType.CARD)


class PaymentRequestIdDestination(RequestDestination):
    id: str

    def __init__(self):
        super().__init__(PaymentDestinationType.ID)


class PaymentRequestTokenDestination(RequestDestination):
    token: str
    billing_address: Address
    phone: Phone

    def __init__(self):
        super().__init__(PaymentDestinationType.TOKEN)


# Request
class PaymentRequest:
    source: RequestSource
    amount: int
    currency: Currency
    payment_type: PaymentType
    merchant_initiated: bool
    reference: str
    description: str
    capture: bool
    capture_on: datetime
    customer: CustomerRequest
    billing_descriptor: BillingDescriptor
    shipping: ShippingDetails
    previous_payment_id: str
    risk: RiskRequest
    success_url: str
    failure_url: str
    payment_ip: str
    three_ds: ThreeDsRequest
    recipient: PaymentRecipient
    metadata: dict
    processing: ProcessingSettings


class PayoutRequest:
    destination: RequestDestination
    amount: int
    fund_transfer_type: FundTransferType
    currency: Currency
    payment_type: PaymentType
    reference: str
    description: str
    capture: bool
    capture_on: datetime
    customer: CustomerRequest
    billing_descriptor: BillingDescriptor
    shipping: ShippingDetails
    three_ds: ThreeDsRequest
    previous_payment_id: str
    risk: RiskRequest
    success_url: str
    failure_url: str
    payment_ip: str
    recipient: PaymentRecipient
    metadata: dict
    processing: dict


# Captures
class CaptureRequest:
    amount: int
    reference: str
    metadata: dict



from __future__ import absolute_import

from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.customers.customers import CustomerRequest


class CustomersClient(Client):
    __CUSTOMERS_PATH = 'customers'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY)

    def get(self, customer_id: str):
        return self._api_client.get(self.build_path(self.__CUSTOMERS_PATH, customer_id),
                                    self._sdk_authorization())

    def create(self, customer_request: CustomerRequest):
        return self._api_client.post(self.__CUSTOMERS_PATH,
                                     self._sdk_authorization(),
                                     customer_request)

    def update(self, customer_id: str, customer_request: CustomerRequest):
        return self._api_client.patch(self.build_path(self.__CUSTOMERS_PATH, customer_id),
                                      self._sdk_authorization(),
                                      customer_request)

    def delete(self, customer_id: str):
        return self._api_client.delete(self.build_path(self.__CUSTOMERS_PATH, customer_id),
                                       self._sdk_authorization())



from checkout_sdk.api_client import ApiClient
from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.checkout_configuration import CheckoutConfiguration
from checkout_sdk.client import Client
from checkout_sdk.financial.financial import FinancialActionsQuery


class FinancialClient(Client):
    __FINANCIAL_ACTIONS_PATH = 'financial-actions'

    def __init__(self, api_client: ApiClient, configuration: CheckoutConfiguration):
        super().__init__(api_client=api_client,
                         configuration=configuration,
                         authorization_type=AuthorizationType.SECRET_KEY_OR_OAUTH)

    def query(self, query: FinancialActionsQuery):
        return self._api_client.get(self.__FINANCIAL_ACTIONS_PATH, self._sdk_authorization(), query)



from checkout_sdk.common.common import ShippingInfo
from checkout_sdk.common.enums import PaymentSourceType, Country, Currency


class KlarnaProduct:
    name: str
    quantity: int
    unit_price: int
    tax_rate: int
    total_amount: int
    total_tax_amount: int


class Klarna:
    description: str
    products: list  # KlarnaProduct
    shipping_info: list  # ShippingInfo
    shipping_delay: int


class CreditSessionRequest:
    purchase_country: Country
    currency: Currency
    locale: str
    amount: int
    tax_amount: int
    products: list  # KlarnaProduct


class OrderCaptureRequest:
    type: PaymentSourceType = PaymentSourceType.KLARNA
    amount: int
    reference: str
    metadata: dict
    klarna: Klarna
    shipping_info: ShippingInfo
    shipping_delay: int



from __future__ import absolute_import

import logging

from checkout_sdk.authorization_type import AuthorizationType
from checkout_sdk.utils import map_to_http_metadata


class CheckoutException(Exception):

    def __init__(self, message=None):
        if message is None:
            message = ""
        super().__init__(message)


class CheckoutArgumentException(CheckoutException):
    pass


class CheckoutAuthorizationException(CheckoutException):

    def __init__(self, message=None):
        super().__init__(message)

    @staticmethod
    def invalid_authorization(authorization_type: AuthorizationType):
        raise CheckoutAuthorizationException(
            'Operation requires ' + authorization_type.name + ' authorization type')

    @staticmethod
    def invalid_key(key_type: AuthorizationType):
        raise CheckoutAuthorizationException('%s is required for this operation' % key_type.name)


class CheckoutApiException(CheckoutException):
    http_metadata: dict
    request_id: str
    error_details: list
    error_type: str

    def __init__(self, response):
        self.http_metadata = map_to_http_metadata(response)
        self.request_id = None
        self.error_details = None
        self.error_type = None

        if response.text:
            try:
                payload = response.json()
                self.request_id = payload.get('request_id')
                self.error_details = payload.get('error_codes')
                self.error_type = payload.get('error_type')
            except (ValueError, KeyError, TypeError) as e:
                logging.error("Failed to parse response JSON payload: %s", e)

        if not self.request_id:
            self.request_id = response.headers.get('Cko-Request-Id')

        super().__init__('The API response status code ({}) does not indicate success.'
                         .format(response.status_code))
